using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Http;
using System.Diagnostics;
using System.Threading;
using System.Net.Http.Headers;
using System.IO;
using Newtonsoft.Json.Linq;
using System.Xml.Linq;

namespace TC_CXModel_Backup
{
    class Program
    {
        static string accountNumber = "";
        static string apiKey = "";

        static void Main(string[] args)
        {

            Console.WriteLine("Initiating Backup...");

            var task2 = exportTestCases();
            task2.Wait();

            var task3 = exportBlocks();
            task3.Wait();

            var task4 = exportDatasets();
            task4.Wait();

            var task5 = exportCXModels();
            task5.Wait();

            var task6 = exportCampaigns();
            task6.Wait();

            // Keep the console window open in debug mode.
            //      Console.WriteLine("Press any key to exit.");
            //    Console.ReadKey();

        }

         static async Task exportDatasets()
        {
            Console.WriteLine("Datasets Export initiated...");

            List<string> datasetIDs = new List<string>();
            List<string> datasetNames = new List<string>();
            List<JObject> datasetsReturned = new List<JObject>();

            var baseAddress = new Uri("https://www.cyaraportal.us/cyarawebapi/v3.0/");

            StringBuilder listResponse = new StringBuilder();

            int pageNumber = 1;
            bool hasMorePages = true;

            while (hasMorePages)
            {
                Console.WriteLine("Processing Dataset page: " + pageNumber);
                using (var httpClient = new HttpClient { BaseAddress = baseAddress })
                {
                    httpClient.DefaultRequestHeaders.TryAddWithoutValidation("accept", "application/json");
                    httpClient.DefaultRequestHeaders.Add("Authorization", apiKey);

                    using (var response = await httpClient.GetAsync("accounts/" + accountNumber + "/datasets?pageSize=1000&pageNo=" + pageNumber))
                    {
                        string responseData = await response.Content.ReadAsStringAsync();
                        listResponse.Append(responseData);
                    }

                    string listResponseString = listResponse.ToString();
                    if (listResponseString.Contains("Invalid credentials"))
                    {
                        Console.WriteLine("ERROR: Invalid credentials upon export. Please check your key and token.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                    else if (listResponseString.Contains("Failed to validate request"))
                    {
                        Console.WriteLine("Export Failed. Please read the error output above.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                }
                
                JObject datasetsJson = JObject.Parse(listResponse.ToString());
                
                string nextPage = (string)datasetsJson.GetValue("nextPage");
                if (String.IsNullOrEmpty(nextPage))
                {
                    hasMorePages = false;
                }
                else
                {
                    pageNumber++;
                }
                JArray datasetsArray = (JArray)datasetsJson["results"];
                foreach (JObject dataset in datasetsArray.ToList())
                {
                    datasetsReturned.Add(dataset);
                    string id = (string)dataset.GetValue("datasetId");
                    datasetIDs.Add(id);
                    string name = (string)dataset.GetValue("name");
                    datasetNames.Add(name);
                }


                listResponse = new StringBuilder();
            }


            Console.WriteLine("Finished pulling list of Datasets. Starting to export...");

            DateTime thisDay = DateTime.Today;
            string myDate = thisDay.ToString("yyyyMMdd");

            string dateTodayPath = Directory.GetCurrentDirectory() + "\\" + myDate + "\\Datasets";
            System.IO.Directory.CreateDirectory(dateTodayPath);

            Console.WriteLine("Total Datasets: " + datasetIDs.Count);

            for (int x = 0; x < datasetIDs.Count; x++)
            {
                JArray datasetRowsJA = new JArray();

                Console.WriteLine("Exporting Rows for Dataset ID: " + datasetIDs[x]);

                StringBuilder datasetResponse = new StringBuilder();

                pageNumber = 1;
                hasMorePages = true;

                while (hasMorePages)
            {
                Console.WriteLine("Processing Dataset Row page: " + pageNumber);
                using (var httpClient = new HttpClient { BaseAddress = baseAddress })
                {
                    httpClient.DefaultRequestHeaders.TryAddWithoutValidation("accept", "application/json");
                    httpClient.DefaultRequestHeaders.Add("Authorization", apiKey);

                    using (var response = await httpClient.GetAsync("accounts/" + accountNumber + "/datasets/" + datasetIDs[x] + "/rows?pageSize=1000&pageNo=" + pageNumber))
                    {
                        string responseData = await response.Content.ReadAsStringAsync();
                        listResponse.Append(responseData);
                    }

                    string listResponseString = listResponse.ToString();
                    if (listResponseString.Contains("Invalid credentials"))
                    {
                        Console.WriteLine("ERROR: Invalid credentials upon export. Please check your key and token.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                    else if (listResponseString.Contains("Failed to validate request"))
                    {
                        Console.WriteLine("Export Failed. Please read the error output above.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                }
                
                JObject datasetRowsJson = JObject.Parse(listResponse.ToString());
                
                string nextPage = (string)datasetRowsJson.GetValue("nextPage");
                if (String.IsNullOrEmpty(nextPage))
                {
                    hasMorePages = false;
                }
                else
                {
                    pageNumber++;
                }
                JArray datasetsRowsArray = (JArray)datasetRowsJson["results"];
                foreach (JObject datasetRow in datasetsRowsArray.ToList())
                {
                    datasetRowsJA.Add(datasetRow);
                }


                listResponse = new StringBuilder();
            } //end there being more pages

JObject datasetStraight = datasetsReturned[x];
datasetStraight.Add("rows", datasetRowsJA);

                string datasetFileName = datasetNames[x] + "_" + datasetIDs[x] + ".xml";

                string uniqueJson = System.IO.Path.Combine(dateTodayPath, datasetFileName);

                File.WriteAllText(uniqueJson, datasetStraight.ToString());
                //Environment.Exit(1);
            }
        }

        static async Task exportBlocks()
        {
            Console.WriteLine("Blocks Export initiated...");

            List<string> blockIDs = new List<string>();

            var baseAddress = new Uri("https://www.cyaraportal.us/cyarawebapi/v3.0/");

            StringBuilder listResponse = new StringBuilder();

            int pageNumber = 1;
            bool hasMorePages = true;

            while (hasMorePages)
            {
                Console.WriteLine("Processing Block page: " + pageNumber);
                using (var httpClient = new HttpClient { BaseAddress = baseAddress })
                {
                    httpClient.DefaultRequestHeaders.TryAddWithoutValidation("accept", "application/json");
                    httpClient.DefaultRequestHeaders.Add("Authorization", apiKey);

                    using (var response = await httpClient.GetAsync("accounts/" + accountNumber + "/blocks?pageSize=1000&pageNo=" + pageNumber))
                    {
                        string responseData = await response.Content.ReadAsStringAsync();
                        listResponse.Append(responseData);
                    }

                    string listResponseString = listResponse.ToString();
                    if (listResponseString.Contains("Invalid credentials"))
                    {
                        Console.WriteLine("ERROR: Invalid credentials upon export. Please check your key and token.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                    else if (listResponseString.Contains("Failed to validate request"))
                    {
                        Console.WriteLine("Export Failed. Please read the error output above.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                }

                JObject blocksJson = JObject.Parse(listResponse.ToString());
                string nextPage = (string)blocksJson.GetValue("nextPage");
                if (String.IsNullOrEmpty(nextPage))
                {
                    hasMorePages = false;
                }
                else
                {
                    pageNumber++;
                }
                JArray blocksArray = (JArray)blocksJson["results"];
                foreach (JObject block in blocksArray.ToList())
                {
                    string id = (string)block.GetValue("blockId");
                    blockIDs.Add(id);
                    Console.WriteLine(id);
                }


                listResponse = new StringBuilder();
            }
            Console.WriteLine("Finished pulling list of Blocks. Starting to export...");

            DateTime thisDay = DateTime.Today;
            string myDate = thisDay.ToString("yyyyMMdd");

            string dateTodayPath = Directory.GetCurrentDirectory() + "\\" + myDate + "\\Blocks";
            System.IO.Directory.CreateDirectory(dateTodayPath);

            for (int x = 0; x < blockIDs.Count; x++)
            {
                Console.WriteLine("Export Block: " + blockIDs[x]);

                StringBuilder blockResponse = new StringBuilder();

                using (var httpClient = new HttpClient { BaseAddress = baseAddress })
                {
                    httpClient.DefaultRequestHeaders.TryAddWithoutValidation("accept", "application/xml");
                    httpClient.DefaultRequestHeaders.Add("Authorization", apiKey);

                    using (var response = await httpClient.GetAsync("accounts/" + accountNumber + "/blocks/cyaraxml?blockIds=" + blockIDs[x]))
                    {
                        string responseData = await response.Content.ReadAsStringAsync();
                        blockResponse.Append(responseData);
                    }
                }

                XDocument xmlDoc = XDocument.Parse(blockResponse.ToString());

                Console.WriteLine(blockResponse.ToString());
                string blockName = getElement(xmlDoc, "BlockName").Value;
                string invalid = new string(Path.GetInvalidFileNameChars()) + new string(Path.GetInvalidPathChars());

                foreach (char c in invalid)
                {
                    blockName = blockName.Replace(c.ToString(), "");
                }

                string blockPath = getElement(xmlDoc, "FolderPath").Value;

                string dateTodayPathBak = dateTodayPath;
                if (!blockPath.Equals(""))
                {
                    dateTodayPath = dateTodayPath + "\\" + blockPath;
                    System.IO.Directory.CreateDirectory(dateTodayPath);
                }


                string blockFileName = blockName + "_" + blockIDs[x] + ".xml";

                string uniqueXML = System.IO.Path.Combine(dateTodayPath, blockFileName);

                File.WriteAllText(uniqueXML, blockResponse.ToString());
                dateTodayPath = dateTodayPathBak;
            }
        }

        static async Task exportCampaigns()
        {
            Console.WriteLine("Campaigns Export initiated...");

            List<string> campaignIDs = new List<string>();

            var baseAddress = new Uri("https://www.cyaraportal.us/cyarawebapi/v3.0/");

            StringBuilder listResponse = new StringBuilder();

            int pageNumber = 1;
            bool hasMorePages = true;

            while (hasMorePages)
            {
                Console.WriteLine("Processing Campaign page: " + pageNumber);
                using (var httpClient = new HttpClient { BaseAddress = baseAddress })
                {
                    httpClient.DefaultRequestHeaders.TryAddWithoutValidation("accept", "application/json");
                    httpClient.DefaultRequestHeaders.Add("Authorization", apiKey);

                    using (var response = await httpClient.GetAsync("accounts/" + accountNumber + "/campaigns?pageSize=1000&pageNo=" + pageNumber))
                    {
                        string responseData = await response.Content.ReadAsStringAsync();
                        listResponse.Append(responseData);
                    }

                    string listResponseString = listResponse.ToString();
                    if (listResponseString.Contains("Invalid credentials"))
                    {
                        Console.WriteLine("ERROR: Invalid credentials upon export. Please check your key and token.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                    else if (listResponseString.Contains("Failed to validate request"))
                    {
                        Console.WriteLine("Export Failed. Please read the error output above.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                }
                
                JObject campaignsJson = JObject.Parse(listResponse.ToString());
                
                string nextPage = (string)campaignsJson.GetValue("nextPage");
                if (String.IsNullOrEmpty(nextPage))
                {
                    hasMorePages = false;
                }
                else
                {
                    pageNumber++;
                }
                JArray campainsArray = (JArray)campaignsJson["results"];
                foreach (JObject campaign in campainsArray.ToList())
                {
                    string id = (string)campaign.GetValue("campaignId");
                    campaignIDs.Add(id);
                }


                listResponse = new StringBuilder();
            }


            Console.WriteLine("Finished pulling list of Campaigns. Starting to export...");

            DateTime thisDay = DateTime.Today;
            string myDate = thisDay.ToString("yyyyMMdd");

            string dateTodayPath = Directory.GetCurrentDirectory() + "\\" + myDate + "\\Campaigns";
            System.IO.Directory.CreateDirectory(dateTodayPath);

            Console.WriteLine("Total Campaigns: " + campaignIDs.Count);

             for (int x = 0; x < campaignIDs.Count; x++)
            {
                Console.WriteLine("Running: " + "cyarawebapi/v3.0/accounts/" + accountNumber + "/campaigns/" + campaignIDs[x]);

                StringBuilder campaignResponse = new StringBuilder();

                using (var httpClient = new HttpClient { BaseAddress = baseAddress })
                {

                    httpClient.DefaultRequestHeaders.TryAddWithoutValidation("accept", "application/json");
                    httpClient.DefaultRequestHeaders.Add("Authorization", apiKey);

                    using (var response = await httpClient.GetAsync("accounts/" + accountNumber + "/campaigns/" + campaignIDs[x]))
                    {
                        string responseData = await response.Content.ReadAsStringAsync();
                        campaignResponse.Append(responseData);
                    }
                }
                JObject campaignJson = JObject.Parse(campaignResponse.ToString());

                string campaignName = (string)campaignJson.GetValue("name");
                string invalid = new string(Path.GetInvalidFileNameChars()) + new string(Path.GetInvalidPathChars());

                foreach (char c in invalid)
                {
                    campaignName = campaignName.Replace(c.ToString(), "");
                }
                string dateTodayPathBak = dateTodayPath;

                System.IO.Directory.CreateDirectory(dateTodayPath);


                string campaignFileName = campaignName + "_" + campaignIDs[x] + ".json";

                string uniqueJsonPath = System.IO.Path.Combine(dateTodayPath, campaignFileName);

                File.WriteAllText(uniqueJsonPath, campaignJson.ToString());
            }
        }


       static async Task exportCXModels()
        {
            Console.WriteLine("CXModel Export initiated...");

            List<string> cxModelIDs = new List<string>();
			List<string> modelFolderPaths = new List<string>();

            var baseAddress = new Uri("https://www.cyaraportal.us/cyarawebapi/v3.0/");

            StringBuilder listResponse = new StringBuilder();

            int pageNumber = 1;
            bool hasMorePages = true;

            while (hasMorePages)
            {
                Console.WriteLine("Processing CXModels page: " + pageNumber);
                using (var httpClient = new HttpClient { BaseAddress = baseAddress })
                {
                    httpClient.DefaultRequestHeaders.TryAddWithoutValidation("accept", "application/json");
                    httpClient.DefaultRequestHeaders.Add("Authorization", apiKey);

                    using (var response = await httpClient.GetAsync("accounts/" + accountNumber + "/models?pageSize=1000&pageNo=" + pageNumber))
                    {
                        string responseData = await response.Content.ReadAsStringAsync();
                        listResponse.Append(responseData);
                    }

                    string listResponseString = listResponse.ToString();
                    if (listResponseString.Contains("Invalid credentials"))
                    {
                        Console.WriteLine("ERROR: Invalid credentials upon export. Please check your key and token.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                    else if (listResponseString.Contains("Failed to validate request"))
                    {
                        Console.WriteLine("Export Failed. Please read the error output above.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                }
                
                JObject modelsJson = JObject.Parse(listResponse.ToString());
                
                string nextPage = (string)modelsJson.GetValue("nextPage");
                if (String.IsNullOrEmpty(nextPage))
                {
                    hasMorePages = false;
                }
                else
                {
                    pageNumber++;
                }
                JArray modelsArray = (JArray)modelsJson["results"];
                foreach (JObject model in modelsArray.ToList())
                {
                    string id = (string)model.GetValue("modelId");
                    cxModelIDs.Add(id);
					string folderPath = (string)model.GetValue("folderPath");
					modelFolderPaths.Add(folderPath);
                }


                listResponse = new StringBuilder();
            }


            Console.WriteLine("Finished pulling list of CXModels. Starting to export...");

            DateTime thisDay = DateTime.Today;
            string myDate = thisDay.ToString("yyyyMMdd");

            string dateTodayPath = Directory.GetCurrentDirectory() + "\\" + myDate + "\\CX_Models";
            System.IO.Directory.CreateDirectory(dateTodayPath);

            Console.WriteLine("Total CX Models: " + cxModelIDs.Count);

             for (int x = 0; x < cxModelIDs.Count; x++)
            {
                Console.WriteLine("Running: " + "cyarawebapi/v3.0/accounts/" + accountNumber + "/models/" + cxModelIDs[x] + "?includeDataset=true");

                StringBuilder modelResponse = new StringBuilder();

                using (var httpClient = new HttpClient { BaseAddress = baseAddress })
                {

                    httpClient.DefaultRequestHeaders.TryAddWithoutValidation("accept", "application/json");
                    httpClient.DefaultRequestHeaders.Add("Authorization", apiKey);

                    using (var response = await httpClient.GetAsync("accounts/" + accountNumber + "/models/" + cxModelIDs[x] + "?includeDataset=true"))
                    {
                        string responseData = await response.Content.ReadAsStringAsync();
                        modelResponse.Append(responseData);
                    }
                }
                JObject modelJson = JObject.Parse(modelResponse.ToString());

                string modelName = (string)modelJson.GetValue("name");
                string invalid = new string(Path.GetInvalidFileNameChars()) + new string(Path.GetInvalidPathChars());

                foreach (char c in invalid)
                {
                    modelName = modelName.Replace(c.ToString(), "");
                }

                string dateTodayPathBak = dateTodayPath;

                dateTodayPath = dateTodayPath + modelFolderPaths[x];
                System.IO.Directory.CreateDirectory(dateTodayPath);


                string modelFileName = modelName + "_" + cxModelIDs[x] + ".json";

                string uniqueJsonPath = System.IO.Path.Combine(dateTodayPath, modelFileName);

                File.WriteAllText(uniqueJsonPath, modelJson.ToString());
				
				dateTodayPath = dateTodayPathBak;
            }
        }

        static async Task exportTestCases()
        {
            Console.WriteLine(apiKey);
           // Environment.Exit(1);

            Console.WriteLine("Test Cases Export initiated...");

            List<string> testCaseIDs = new List<string>();

            var baseAddress = new Uri("https://www.cyaraportal.us/cyarawebapi/v3.0/");

            StringBuilder listResponse = new StringBuilder();

            int pageNumber = 1;
            bool hasMorePages = true;

            while (hasMorePages)
            {
                Console.WriteLine("Processing TC page: " + pageNumber);
                using (var httpClient = new HttpClient { BaseAddress = baseAddress })
                {
                    httpClient.DefaultRequestHeaders.TryAddWithoutValidation("accept", "application/json");
                    httpClient.DefaultRequestHeaders.Add("Authorization", apiKey);

                    using (var response = await httpClient.GetAsync("accounts/" + accountNumber + "/testcases?pageSize=1000&pageNo=" + pageNumber))
                    {
                        string responseData = await response.Content.ReadAsStringAsync();
                        listResponse.Append(responseData);
                    }

                    string listResponseString = listResponse.ToString();
                    if (listResponseString.Contains("Invalid credentials"))
                    {
                        Console.WriteLine("ERROR: Invalid credentials upon export. Please check your key and token.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                    else if (listResponseString.Contains("Failed to validate request"))
                    {
                        Console.WriteLine("Export Failed. Please read the error output above.");
                        Console.WriteLine("Press enter to close");
                        Console.ReadLine();
                        Environment.Exit(1);
                    }
                }

                JObject testCasesJson = JObject.Parse(listResponse.ToString());
                string nextPage = (string)testCasesJson.GetValue("nextPage");
                if (String.IsNullOrEmpty(nextPage))
                {
                    hasMorePages = false;
                }
                else
                {
                    pageNumber++;
                }
                JArray testCasesArray = (JArray)testCasesJson["results"];
                foreach (JObject testCase in testCasesArray.ToList())
                {
                    //Console.WriteLine(testCase);
                    string id = (string)testCase.GetValue("testCaseId");
                    testCaseIDs.Add(id);
                    Console.WriteLine(id);
                }


                listResponse = new StringBuilder();
            }

            Console.WriteLine("Finished pulling list of Test Cases. Starting to export...");

            DateTime thisDay = DateTime.Today;
            string myDate = thisDay.ToString("yyyyMMdd");

            string dateTodayPath = Directory.GetCurrentDirectory() + "\\" + myDate + "\\Test Cases";
            System.IO.Directory.CreateDirectory(dateTodayPath);

            for (int x = 0; x < testCaseIDs.Count; x++)
            {
                Console.WriteLine("Export TestCase: " + testCaseIDs[x]);

                StringBuilder testCaseResponse = new StringBuilder();

                using (var httpClient = new HttpClient { BaseAddress = baseAddress })
                {
                    httpClient.DefaultRequestHeaders.TryAddWithoutValidation("accept", "application/xml");
                    httpClient.DefaultRequestHeaders.Add("Authorization", apiKey);

                    using (var response = await httpClient.GetAsync("accounts/" + accountNumber + "/testcases/cyaraxml?testCaseIds=" + testCaseIDs[x] + "&includeDataDriven=true&includeServices=true&exportFormat=XML"))
                    {
                        string responseData = await response.Content.ReadAsStringAsync();
                        testCaseResponse.Append(responseData);
                    }
                }

                XDocument xmlDoc = XDocument.Parse(testCaseResponse.ToString());
                
                string testCaseName = getElement(xmlDoc, "TestCaseName").Value;
                string invalid = new string(Path.GetInvalidFileNameChars()) + new string(Path.GetInvalidPathChars());

                foreach (char c in invalid)
                {
                    testCaseName = testCaseName.Replace(c.ToString(), "");
                }

                string testCasePath = getElement(xmlDoc, "FolderPath").Value;

                if (!testCasePath.StartsWith("zzzCrawler\\")) {

                    string dateTodayPathBak = dateTodayPath;
                    if (!testCasePath.Equals(""))
                    {
                        dateTodayPath = dateTodayPath + "\\" + testCasePath;
                        System.IO.Directory.CreateDirectory(dateTodayPath);
                    }


                    string testCaseFileName = testCaseName + "_" + testCaseIDs[x] + ".xml";

                    string uniqueXML = System.IO.Path.Combine(dateTodayPath, testCaseFileName);

                    File.WriteAllText(uniqueXML, testCaseResponse.ToString());
                    dateTodayPath = dateTodayPathBak;
                }
            }
        }

        static XElement getElement(XDocument doc, string elementName)
        {
            foreach (XNode node in doc.DescendantNodes())
            {
                if (node is XElement)
                {
                    XElement element = (XElement)node;
                    if (element.Name.LocalName.Equals(elementName))
                    {
                        return element;
                    }
                }
            }
            return null;
        }
    }
}