#v2.1.2.0 2021-01-29 Bug fixes.
#v2.1.1.0 2021-01-27 Improved calls made calculation for Stepped test. Added check for banned tags. Improved comments.
#v2.0.0.0 2021-01-24 Added Stairstepping. Added ANI spoofing. Added LogInfo CallID. Added Speech reply check. Bug fixes.
	#Improved CAPS and Port calculations. Improved Naming convention. Improved Description. Added configuration options.
#v1.0.1.0 2020-06-17 Bug fixes and logic changes.
#v1.0.0.0 2020-06-04 Initial release


<#
Welcome to the fully automated load test campaign generator.
This program will handle both your straight shot load test and
stepped load test, two of Cyara's most popular Performance Tests.

And when we say automate, we mean automate. Fill out the
configurables located below, and the programs takes care of the rest.
Among other validations, it will:

	Verify your test cases are properly setup for load testing

	Automatically build your Cruncher and Cruncher Lite Campaigns

	Calculate the most efficient setup for your test

	Check the math of your setup to ensure you have a successful load test

	Create your load test run sheet

	Follow all of Cyara's Best Practices for Performance Tests


We also try to maximize your load test window, which is typically 4 hours.
As part of this, we recognize you may want to run multiple tests
inside that 4 hour window. These multiple different tests are called "segments."


If you've used this before and are using it again for a new load test,
make sure you remove any old RunSheet.csv and Segments.log files.

Once everything is configured, right click on this file and choose Run with PowerShell.
You can also run this within PowerShell ISE.

Formulas based on https://voipbilling.wordpress.com/2010/01/13/number-of-simultaneous-calls-vs-caps/

This is not an official Cyara product and is not officially supported.
It is used for demonstration and training purposes.
For assistance, contact chris.ryan@cyara.com
@author Chris Ryan
#>


[Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls -bor [Net.SecurityProtocolType]::Tls11 -bor [Net.SecurityProtocolType]::Tls12

##############################
# START OF CONFIGURABLES #
##############################

$apiKey = "ApiKey YOUR_KEY:YOUR_TOKEN" #Change to your ApiKey Key:Token
$testDate = "2021-02-28" #Change to your load test date in format yyyy-MM-dd
$portal = "US" #Can be AU, UK, or US
$accountNum = 0 #Change to your account number
$crPlan = 0 #Change to your Cruncher plan ID
$clPlan = 0 #Change to your Cruncher Lite plan ID
$segmentPorts = 0 #Change to how many ports you want your load test to be
$segmentDurationMinutes = 0 #Change to how many minutes you want your load test to be
$requiredSegmentCalls = 0 #Set to 0 if there is no required total call count
$maxOrRequestedCAPS = 0 #Specify how many CAPS you can or should use
$capsIsCeilingOrRequirement = "Required"; #Can be "Ceiling" or Required". Ceiling means physically can't go higher (the Do Not Exceed). Requirement means customer must maintain this.
$testCaseDistributionProfile = "RoundRobin" #Can be "RoundRobin" or "UserDefined"
$loadIncrement = "Stepped" #Can be "Linear" or "Stepped"
<#
Keep aniSpoofTestCases empty if you don't want ANI spoofing or don't want US ANIs.
Set to true or your 3 digit area code if you do.
Using true will give a random area code for your test case(s).
Using your 3 digit area code will create random ANIs with that area code.
If your test case is already ANI spoofed in any way, this will not overwrite that.
If your test case is being data driven (but not for your ANI), this will
	ANI spoof the test case with a single, random, hardcoded ANI such as 18005551234;From:16505498522
#>
$aniSpoofTestCases = "true"
<#
Change to the test cases you want to use in your campaigns.
You can add or remove any test cases you want, you just need to have at least one.
If you're using RoundRobin, change the ProbabilityPct to 100 for each test case.
#>
$testCases =
[xml]"<TestCases>
	<TestCase>
		<TestCaseID>487964</TestCaseID>
		<TestCaseMinuteDuration>0.5</TestCaseMinuteDuration>
		<ProbabilityPct>100</ProbabilityPct>
	</TestCase>
</TestCases>"

##############################
# END OF CONFIGURABLES #
##############################

# Logs to the runsheet the campaign the program created
Function addCampaignToRunSheet
{
	Param ([string]$logstring)

	$Logfile = "RunSheet.csv"

	Write-Host $logString

	Add-content $Logfile -value "$logstring"

}


#Uses the 3.0 API to create a campaign based on the json body passed in
Function createCampaign
{
	Param ([string]$body)

	$headers = New-Object "System.Collections.Generic.Dictionary[[String],[String]]"
	$headers.Add("Authorization", $apiKey)
	$headers.Add("Accept", "applicaton/json")

	$resource = "https://www.cyaraportal.$portalDomain/CyaraWebApi/v3.0/accounts/$accountNum/campaigns"

	$return = Invoke-RestMethod -Method POST -Uri $resource -Headers $headers -ContentType "application/json" -Body $body

	return $return
}


#Uses the 3.0 API to fetch a specific plan's details on the account.
Function getPlanDetails
{
	Param ([string]$planID)

	$headers = New-Object "System.Collections.Generic.Dictionary[[String],[String]]"
	$headers.Add("Authorization", $apiKey)
	$headers.Add("Accept", "applicaton/json")

	$resource = "https://www.cyaraportal.$portalDomain/CyaraWebApi/v3.0/accounts/$accountNum/plans/$planID"

	$return = Invoke-RestMethod -Method GET -Uri $resource -Headers $headers

	return $return
}


#Uses the 3.0 API to export a specified test case as Cyara XML
Function exportTestCase
{
	Param ([string]$testCaseID)

	$headers = New-Object "System.Collections.Generic.Dictionary[[String],[String]]"
	$headers.Add("Authorization", $apiKey)
	$headers.Add("Accept", "applicaton/json")

	$resource = "https://www.cyaraportal.$portalDomain/CyaraWebApi/v3.0/accounts/$accountNum/testcases/cyaraxml?testCaseIds=$testCaseID&recursive=false&includeDataDriven=true&includeServices=true&download=false&blockOptions=IncludeBlocks"

	$return = Invoke-RestMethod -Method GET -Uri $resource -Headers $headers

	return $return
}


#Uses the 3.0 API to import a specified test case from Cyara XML
Function importTestCase
{
	Param ([string]$cyaraXmlBody)

	$headers = New-Object "System.Collections.Generic.Dictionary[[String],[String]]"
	$headers.Add("Authorization", $apiKey)
	$headers.Add("Accept", "applicaton/json")

	$resource = "https://www.cyaraportal.$portalDomain/CyaraWebApi/v3.0/accounts/$accountNum/testcases/import"

	$return = Invoke-RestMethod -Method POST -Uri $resource -Headers $headers -Body $cyaraXmlBody

	return $return
}


#Logs the passed message into the log file. Prepends a date and time
Function logMessage
{
	Param ([string]$logstring)

	$Logfile = "Segments.log"

	$timestamp = Get-Date -Format "yyyy-MM-dd HH:mm"

	Write-Host $logString

	Add-content $Logfile -value "$timestamp $logstring"

}



$test = $testCases.OuterXml
logMessage "`$apiKey = `"$apiKey`"
`$portal = `"$portal`"
`$accountNum = $accountNum
`$crPlan = $crPlan
`$clPlan = $clPlan
`$segmentPorts = $segmentPorts
`$segmentDurationMinutes = $segmentDurationMinutes
`$requiredSegmentCalls = $requiredSegmentCalls
`$maxOrRequestedCAPS = $maxOrRequestedCAPS
`$testCaseDistributionProfile = `"$testCaseDistributionProfile`"
`$loadIncrement = `"$loadIncrement`"
`$aniSpoofing = `$aniSpoofing
`$testCases = [xml]`"$test`"
"


#Get the current date in the API-friendly format
$time = Get-Date -format s
$runDate = $time + "z"


#Return as an array the characters A-Z. Used in the campaign name and runsheet to show which campaigns are started at the same time
$AZ = [char[]](65..90)



#Create the correct domain for the portal chosen base on what the user specified
$portalDomain = ""

if ($portal -eq "AU") {
	$portalDomain = "com"
}
elseif ($portal -eq "UK") {
	$portalDomain = "co.uk"
}
elseif ($portal -eq "US") {
	$portalDomain = "us"
}

#Variables to help validate and build test case json
$totalTestCases = 0
$averageTestCaseMinuteDuration = 0
$totalProbability = 0
$testCaseJson = "`"testCases`": ["

<#
Start working out the math for the avergagetest case duration
and building the test case json used when building campaigns with the API
#>
foreach ($testCase in $testCases.TestCases.TestCase)
{
	logMessage "testCase.TestCaseID: $testCase.TestCaseID"
	$totalTestCases = $totalTestCases + 1

	#The user specified their probability percentages, add pct to counter and add averaged duration to counter
	if ($testCaseDistributionProfile -eq "UserDefined") {
		$totalProbability = $totalProbability + $testCase.ProbabilityPct
		$averageTestCaseMinuteDuration = $averageTestCaseMinuteDuration + ([double]$testCase.TestCaseMinuteDuration * ([double]$testCase.ProbabilityPct * .01))
	}
	else { #The user chose RoundRobin, so just add the duration to counter. It will be averaged after the loop
		$averageTestCaseMinuteDuration = $averageTestCaseMinuteDuration + [double]$testCase.TestCaseMinuteDuration
	}

	#For this test case, build the json that will be used in campaign creation
	$tcPct = $testCase.ProbabilityPct
	$tcID = $testCase.TestCaseID
	$testCaseJson = "$testCaseJson {
	`"probability`": $tcPct,
	`"testCaseId`": $tcID
		},"
}

#Remove the last, unnecessary comma from the test case json, then add the closing square bracket
$testCaseJson = $testCaseJson.Substring(0,$testCaseJson.Length-1)
$testCaseJson = "$testCaseJson]"
logMessage "testCaseJson $testCaseJson"

#Here is where we average the test case duration if the user chose RoundRobin
if ($testCaseDistributionProfile -eq "RoundRobin") {
	$averageTestCaseMinuteDuration = $averageTestCaseMinuteDuration / $totalTestCases
}

<#
Perform the calculation of how many calls we expect to make with this setup.
Only applied to Linear
If the user is doing a Stepped test, this gets recalculated in the Stepped calculations.
#>
$expectedSegmentCalls = [Math]::Floor([decimal]($segmentDurationMinutes/$averageTestCaseMinuteDuration)) * $segmentPorts
logMessage "expectedSegmentCalls $expectedSegmentCalls"


#Set how many CAPS our campaign will be configured for to what the user specified
$usedCAPS = $maxOrRequestedCAPS

#Calculate how many CAPS the setup will actually maintain
$minimumCAPS = [math]::Round($segmentPorts / 0.8 / $averageTestCaseMinuteDuration / 0.6 / 100, 2)

#The CAPS is not a hard requirement by the customer, so we can use minimum needed
if ($capsIsCeilingOrRequirement -eq "Ceiling") { 
	$usedCAPS = $minimumCAPS
}

<#
Calculate how many ports the setup will actually be able to maintain
0.8 represent the decimal percentage for the Answer-Seizure Ratio.
In telecom, an attempted call is termed a seizure.

The 0.6 represent the conversion to time, because there are 60 seconds in a minute
#>
$calculatedSupportedPorts = 0.8 * $averageTestCaseMinuteDuration * $usedCAPS * 0.6 * 100

#Calculate the actual duration our test case needs to be to support the ports and CAPS
$minimumAvgTestCaseMinuteDuration = [math]::round(($segmentPorts / 0.8 / $usedCAPS / 0.6 / 100), 2)

<#
If the user specified a certain number of calls to be placed, this checks
how long our test case needs to be, and how many ports we need with the current
setup. Used later on to check if the user's setup actually works.

Only applies to Linear. Stepped is recalculated in the Stepped calculations.
#>
$minimumSegCallsAvgTestCaseDuration = 1
$minimumSegmentPorts = 1
if ($requiredSegmentCalls -gt 0) {
	$minimumSegCallsAvgTestCaseDuration = 1 / ($requiredSegmentCalls / $segmentPorts / $segmentDurationMinutes)
	$minimumSegmentPorts = [int][Math]::Ceiling($requiredSegmentCalls * $averageTestCaseMinuteDuration / $segmentDurationMinutes)
}

#Get the Cruncher plan information
$crInfoRaw = getPlanDetails $crPlan
$crInfoString = ConvertTo-Json $crInfoRaw
$crPlanJson = $crInfoString | ConvertFrom-Json
#$crPlanJson | Get-Member
$crMaxPorts = $crPlanJson.concurrency
$crMaxCAPS = $crPlanJson.settings.maximumAps

#Get the Cruncher Lite plan information
$clInfoRaw = getPlanDetails $clPlan
$clInfoString = ConvertTo-Json $clInfoRaw
$clPlanJson = $clInfoString | ConvertFrom-Json
#$clPlanJson | Get-Member
$clMaxPorts = $clPlanJson.concurrency
$clMaxCAPS = $clPlanJson.settings.maximumAps

$combinedPlanCAPS = $crMaxCAPS + $clMaxCAPS
$combinedPlanPorts = $crMaxPorts + $clMaxPorts

logMessage "averageTestCaseMinuteDuration: $averageTestCaseMinuteDuration"
logMessage "expectedSegmentCalls: $expectedSegmentCalls"
logMessage "calculatedSupportedPorts: $calculatedSupportedPorts"
logMessage "minimumCAPS: $minimumCAPS"
logMessage "usedCAPS: $usedCAPS"
logMessage "minimumAvgTestCaseMinuteDuration: $minimumAvgTestCaseMinuteDuration"
logMessage "minimumSegCallsAvgTestCaseDuration: $minimumSegCallsAvgTestCaseDuration"
logMessage "minimumSegmentPorts: $minimumSegmentPorts"
logMessage "crMaxPorts: $crMaxPorts"
logMessage "crMaxCAPS: $crMaxCAPS"
logMessage "clMaxPorts: $clMaxPorts"
logMessage "clMaxCAPS: $clMaxCAPS"

<#
From our log file, scrape how many segments there already are.
This is used to determine if we need to make a new header row in RunSheets.csv
Every run of this program generates a log file containing "Segment created"
We also use the count to determine what to call this campaign, and how to add to the run sheet
#>
$segmentCount = (Select-String -Path "Segments.log" -Pattern "Segment created" -AllMatches).Matches.Count
$segmentCount = $segmentCount + 1

$checksCount = (Select-String -Path "Segments.log" -Pattern "Checks Started" -AllMatches).Matches.Count
if ($checksCount -ge 1) {
	addCampaignToRunSheet "Segment Run #,Step Segment,Run Description,Max Conc Calls,Ramp-Up Time (s),CAPS,Campaign Duration (m),Est. Calls"
}
logMessage "Program Started";

#Array of every valid area code. Some carriers and systems don't accept invalid area codes
$areaCodes = @(205, 251, 256, 334, 659, 938, 907, 480, 520, 602, 623, 928, 479, 501, 870, 209, 213, 279, 310, 323, 341, 408, 415, 424, 442, 510, 530, 559, 562, 619, 626, 628, 650, 657, 661, 669, 707, 714, 747, 760, 805, 818, 820, 831, 840, 858, 909, 916, 925, 949, 951, 303, 719, 720, 970, 203, 475, 860, 959, 302, 202, 239, 305, 321, 352, 386, 407, 448, 561, 689, 727, 754, 772, 786, 813, 850, 863, 904, 941, 954, 229, 404, 470, 478, 678, 706, 762, 770, 912, 808, 208, 986, 217, 224, 309, 312, 331, 447, 618, 630, 708, 773, 779, 815, 847, 872, 219, 260, 317, 463, 574, 765, 812, 930, 319, 515, 563, 641, 712, 316, 620, 785, 913, 270, 364, 502, 606, 859, 225, 318, 337, 504, 985, 207, 240, 301, 410, 443, 667, 339, 351, 413, 508, 617, 774, 781, 857, 978, 231, 248, 269, 313, 517, 586, 616, 734, 810, 906, 947, 989, 218, 320, 507, 612, 651, 763, 952, 228, 601, 662, 769, 314, 417, 573, 636, 660, 816, 406, 308, 402, 531, 702, 725, 775, 603, 201, 551, 609, 640, 732, 848, 856, 862, 908, 973, 505, 575, 212, 315, 332, 347, 516, 518, 585, 607, 631, 646, 680, 716, 718, 838, 845, 914, 917, 929, 934, 252, 336, 704, 743, 828, 910, 919, 980, 984, 701, 216, 220, 234, 326, 330, 380, 419, 440, 513, 567, 614, 740, 937, 405, 539, 580, 918, 458, 503, 541, 971, 215, 223, 267, 272, 412, 445, 484, 570, 610, 717, 724, 814, 878, 401, 803, 839, 843, 854, 864, 605, 423, 615, 629, 731, 865, 901, 931, 210, 214, 254, 281, 325, 346, 361, 409, 430, 432, 469, 512, 682, 713, 726, 737, 806, 817, 830, 832, 903, 915, 936, 940, 945, 956, 972, 979, 385, 435, 801, 802, 276, 434, 540, 571, 703, 757, 804, 206, 253, 360, 425, 509, 564, 202, 304, 681, 262, 414, 534, 608, 715, 920, 307)


#Check if our test cases are properly built for a load test (automatically fixes where applicable)
foreach ($testCase in $testCases.TestCases.TestCase)
{

	#Get the test case Cyara XML as a string to manipulate
	$curTcID = $testCase.TestCaseID
	$tcCyaraXML = exportTestCase $curTcID
	$rawTcCyaraXML = $tcCyaraXML.OuterXml

	#Check if the test case has a speech reply, something Cruncher Lite doesn't allow
	if ($rawTcCyaraXML -like "*<ReplyExchangeType>Speech</ReplyExchangeType>*") {
		Throw "Speech Error
		You are using a speech reply in test case $curTcID
		Please use an audio file or DTMF reply instead"
	}

	#Check if the test case has any priority recognition tags, something CR and CL don't allow
	if ($rawTcCyaraXML -like "*{Choice*" -or $rawTcCyaraXML -like "*{EndCall*" -or $rawTcCyaraXML -like "*{Optional*") {
		Throw "Invalid Tag Error
		You are using at least one tag that requires priority recognition.
		These tags are not allowed in Cruncher and Cruncher Lite campaigns.
		Please remove any of the following tags entirely in test case $curTcId
		{Choice}
		{EndCall}
		{Optional}"
	}

	#Check for and add SIP ID logging to test case
	if ($rawTcCyaraXML -notlike "*{LogInfo CallID}*") { #TC is missing SIP Call ID logging
		if ($rawTcCyaraXML -like "*<Notes />*") { #TC is has no notes
			$rawTcCyaraXml = $rawTcCyaraXML.replace("<Notes />", "<Notes>{LogInfo CallID}</Notes>")

			importTestCase $rawTcCyaraXML
		}
		else { #TC has notes
			$rawTcCyaraXml = $rawTcCyaraXML.replace("<Notes>", "<Notes>{LogInfo CallID} ")

			importTestCase $rawTcCyaraXML
		}
	}

	#Check for and add ANI spoofing to test case if user wants it and it's not already ANI spoofed
	if ($rawTcCyaraXML -notlike "*;From:*" -and $aniSpoofTestCases.length -gt 2) { #TC is not ani spoofed but needs to be

		#TC is not data driven, so let's data drive the ANI
		if ($rawTcCyaraXML -like "*<DataInputs />*") {
			$dataInputs = "	  <DataInputs>`n
			<ParameterList><Parameter name=`"ANI`" /></ParameterList>`n"

			#Create a random ANI then check if the user wants a specific ANI, and use that instead
			$randomAreaCode = $areaCodes | Get-Random
			if($aniSpoofTestCases -match "^\d+$") {
				$randomAreaCode = $aniSpoofTestCases
			}
			logMessage $randomAreaCode

			#Create a random 7 digits, then incrementally build the 10,000 data driven ANIs in Cyara XML
			$random7 = Get-Random -Minimum 2000000 -Maximum 8888888
			logMessage $random7
			for ($i = 0; $i -lt 10000; $i += 1) {
				$random7++
				$dataInputs = $dataInputs + "		<Scenario><Parameter name=`"ANI`">1$randomAreaCode$random7</Parameter></Scenario>`n"
			}

			$dataInputs = $dataInputs + "</DataInputs>"

			$rawTcCyaraXml = $rawTcCyaraXML.replace("<DataInputs />", $dataInputs)
			$rawTcCyaraXml = $rawTcCyaraXML.replace("</PhoneNo>", ";From:[ANI]</PhoneNo>")

			importTestCase $rawTcCyaraXML
		}
		else { #TC is data driven, so hardcode the ANI spoof
			$randomAreaCode = $areaCodes | Get-Random
			if($aniSpoofTestCases -match "^\d+$") {
				$randomAreaCode = $aniSpoofTestCases
			}
			logMessage $randomAreaCode

			$random7 = Get-Random -Minimum 2000000 -Maximum 8888888
			logMessage $random7

			$rawTcCyaraXml = $rawTcCyaraXML.replace("</PhoneNo>", ";From:1$randomAreaCode$random7</PhoneNo>")
			logMessage $rawTcCyaraXml

			importTestCase $rawTcCyaraXML
		}
	}
}

$runTime =  [timespan]::fromminutes($segmentDurationMinutes)
logMessage "runTime $runTime"

if ($testCaseDistributionProfile -eq "User Defined Probability" -and $totalProbability -ne 100) { #User didn't provide 100% probability total
	Throw "Probability Error
	Your total probability is not equal to 100%. It is " + $totalProbability + "%
	Please adjust your probability to be exactly 100%"
}
elseif ($combinedPlanPorts -lt $segmentPorts) { #There aren't enough ports between CR and CL
	Throw "Insufficient Plan Ports Error
	The allowed ports between the two plans is $combinedPlanPorts
	With the current setup, however, you require at least $segmentPorts Ports"
}
elseif ($calculatedSupportedPorts -lt $segmentPorts) { #CAPS and duration isn't sufficient to achieve desired ports
	$extraMsg = ""
	if ($requiredSegmentCalls -gt 0) {
		$extraMsg = "but cannot exceed $minimumSegCallsAvgTestCaseDuration minutes to achieve your total segment call requirement"
	}
	Throw "Required Ports Error
	Your CAPS and TC Durations will only hit $calculatedSupportedPorts ports which is less than your desired " + $segmentPorts + " ports.
	Consider increasing your CAPS or your test case duration(s).
	With this average test case duration, you would need at least $minimumCAPS CAPS.
	With this CAPS rate, your average test case duration cannot go below $minimumAvgTestCaseMinuteDuration minutes to maintain your port requirement
	$extraMsg"
}
elseif ($requiredSegmentCalls -gt 0 -and $expectedSegmentCalls -lt $requiredSegmentCalls -and $loadIncrement -eq "Linear") { #Calculated Calls Per Segment is less than customer's requested
	$extraMsg = ""
	if ($segmentPorts -gt 0) {
		$extraMsg = "but cannot go below $minimumAvgTestCaseMinuteDuration minutes to maintain your port requirement"
	}

	Throw "Too Few Calls Per Segment Error
	Even at max ports for the entire segment, you will only achieve
	$expectedSegmentCalls calls
	which is less than the required
	$requiredSegmentCalls calls
	Consider decreasing your average test case duration or increasing your ports
	With this average test case duration, you would need at least $minimumSegmentPorts ports
	With this port count, your average test case duration cannot exceed $minimumSegCallsAvgTestCaseDuration minutes to achieve your total segment call requirement
	$extraMsg"
}
elseif ($requiredSegmentCalls -gt 0 -and $expectedSegmentCalls -gt $requiredSegmentCalls * 1.1 -and $expectedSegmentCalls -gt $requiredSegmentCalls + 500 -and $loadIncrement -eq "Linear") { #Calculated Calls Per Segment greatly exceeds the customer's requested
	Throw "Too Many Calls Per Segment Error
	With your setup, you will achieve
	an estimated $expectedSegmentCalls calls
	which is far more than the required
	$requiredSegmentCalls calls.
	Consider increasing your average test case duration or decreasing your ports
	With this average test case duration, you would need at most $minimumSegmentPorts ports
	With this port count, your average test case duration cannot go below $minimumSegCallsAvgTestCaseDuration minutes to achieve your total segment call requirement"
}
elseif ($combinedPlanCAPS -lt $usedCAPS) { #There aren't enough CAPS between CR and CL
	if ($capsIsCeilingOrRequirement -eq "Ceiling") {
		Throw "Insufficient Plan CAPS Error
		The allowed CAPS between the two plans is $combinedPlanCAPS
		With the current setup, however, you require at least $minimumCAPS CAPS
		(which may also less than your ceiling of $maxOrRequestedCAPS CAPS)"
	}
	elseif ($capsIsCeilingOrRequirement -eq "Requirement") {
		Throw "Insufficient Plan CAPS Error
		The allowed CAPS between the two plans is $combinedPlanCAPS
		This is less than your requirement of $usedCAPS CAPS"
	}
}
elseif ($capsIsCeilingOrRequirement -eq "Requirement" -and $minimumCAPS -lt $usedCAPS - 1) { #The CAPS rate is required, but the setup won't maintain that rate
	Throw "Required Ports Error
	Your Ports and TC Durations will only use $minimumCAPS CAPS at load, which is more than 1 CAPS less than your required $usedCAPS CAPS.
	Consider increasing your ports or decreasing your test case duration(s).
	With this setup, you will need to either:
		increase your ports to $calculatedSupportedPorts
		or
		decrease your average test case duration to $minimumAvgTestCaseMinuteDuration minutes"
}

if ($loadIncrement -eq "Linear") {
	("{0:hh\:mm\:ss}" -f $rampUpTime)

	$portsRemaining = $segmentPorts - $crMaxPorts

	$crBody = ""

	if ($portsRemaining -le 0) { #CR has more than the requested segment ports, so check if we can just use that

		$usedCrCAPS = $usedCAPS

		#The CR plan's Max CAPS is less than how many CAPS the user needs. This means we need to combine CR and CL together
		if ($crMaxCAPS -lt $usedCrCAPS) {
			$usedCrCAPS = $crMaxCAPS

			$crPorts = [int][Math]::Floor(0.8 * $averageTestCaseMinuteDuration * $usedCrCAPS * 0.6 * 100)

			if ($crPorts -gt $crMaxPorts) {
				Throw "Insufficient Cruncher Ports
				Based on your Cruncher plan's max ports, trying to use its max CAPS produces a port usage beyond the plan's limit.
				Please either increase your Cruncher plan's port limit or decrease your test case duration(s)."
			}

			$crS = [int][Math]::Ceiling($crPorts / $usedCrCAPS)
			$crRampUpTime = [timespan]::fromseconds($crS)
			("{0:hh\:mm\:ss}" -f $crRampUpTime)

			$remainingPorts = $segmentPorts - $crPorts
			logMessage "remainingPorts $remainingPorts"
			$remainingCAPS = $usedCAPS - $usedCrCAPS
			logMessage "remainingCAPS $remainingCAPS"

			if ($clMaxPorts -lt $remainingPorts) {
				Throw "Missing CruncherLite Ports Error
				Based on your Cruncher plan's max CAPS, $remainingPorts ports were routed to Cruncher Lite
				Your Cruncher Lite plan does not have enough ports for these
				Please either increase your CR CAPS limit or increase your CL ports"
			}
			if ($clMaxCAPS -lt $remainingCAPS) {
				Throw "Missing CruncherLite CAPS Error
				Based on your Cruncher plan's max CAPS, $remainingPorts ports were routed to Cruncher Lite which requires $remainingCAPS CAPS
				Your Cruncher Lite plan does not have enough CAPS for these
				Please either increase your CR CAPS limit or increase your CL CAPS"
			}

			$clS = [int][Math]::Ceiling($remainingPorts / $remainingCAPS)
			$clRampUpTime = [timespan]::fromseconds($clS)
			("{0:hh\:mm\:ss}" -f $clRampUpTime)

			$expectedCRCalls = [Math]::Floor([decimal]($segmentDurationMinutes/$averageTestCaseMinuteDuration)) * $crPorts
			$expectedCRCallsMsg = ""
			if ($requiredSegmentCalls -gt 0) {
				$expectedCRCallsMsg = " out of the required $requiredSegmentCalls calls"
			}

			$crBody = "{
			`"planId`": $crPlan,
			`"active`": false,
			`"channel`": `"Voice`",
			`"name`": `"$testDate Segment $segmentCount.1 $($AZ[$segmentCount-1]) | $crPorts Ports | $usedCrCAPS CAPS | $segmentDurationMinutes Minutes | CR`",
			`"description`": `"CR Plan has enough ports alone but doesn't have enough CAPS based on the user's port count and average test case duration. This campaign uses the max CAPS supported by the CR Plan and thus the max ports that mathematically supports. It should make $expectedCRCalls calls$expectedCRCallsMsg.`",
			`"concurrency`": $crPorts,
			`"maximumAps`": $usedCrCAPS,
			`"rampUpTime`": `"$crRampUpTime`",
			`"runTime`": `"$runTime`",
			`"requestedRunDate`": `"$runDate`",
			`"scheduledRunDate`": `"$runDate`",
			`"testCaseDistributionProfile`": `"$testCaseDistributionProfile`",
			`"planType`": `"Cruncher`",
			$testCaseJson
			}"

			#Create the Cruncher Campaign via the API
			$crReturn = createCampaign $crBody

			addCampaignToRunSheet "$segmentCount.1,$($AZ[$segmentCount-1]),CR Plan has enough ports alone but doesn't have enough CAPS based on the user's port count and average test case duration. This campaign uses the max CAPS supported by the CR Plan and thus the max ports that mathematically supports.,$crPorts,$crS,$usedCrCAPS,$segmentDurationMinutes,$expectedCRCalls"

			$expectedCLCalls = [Math]::Floor([decimal]($segmentDurationMinutes/$averageTestCaseMinuteDuration)) * $clPorts
			$expectedCLCallsMsg = ""
			if ($requiredSegmentCalls -gt 0) {
				$expectedCLCallsMsg = " out of the required $requiredSegmentCalls calls"
			}

			$clBody = "{
			`"planId`": $clPlan,
			`"active`": false,
			`"channel`": `"Voice`",
			`"name`": `"$testDate Segment $segmentCount.2 $($AZ[$segmentCount-1]) | $remainingPorts Ports | $remainingCAPS CAPS | $segmentDurationMinutes Minutes | CL`",
			`"description`": `"The sibling CR campaign can't carry alone all the ports because of the CAPS limitation. This CL campaign picks up the slack. It should make $expectedCLCalls calls$expectedCLCallsMsg.`",
			`"concurrency`": $remainingPorts,
			`"maximumAps`": $remainingCAPS,
			`"rampUpTime`": `"$clRampUpTime`",
			`"runTime`": `"$runTime`",
			`"requestedRunDate`": `"$runDate`",
			`"scheduledRunDate`": `"$runDate`",
			`"testCaseDistributionProfile`": `"$testCaseDistributionProfile`",
			`"planType`": `"CruncherLite`",
			$testCaseJson
			}"

			#Create the Cruncher Lite Campaign via the API
			$clReturn = createCampaign $clBody

			addCampaignToRunSheet "$segmentCount.2,$($AZ[$segmentCount-1]),The sibling CR campaign can't carry alone all the ports because of the CAPS limitation. This CL campaign picks up the slack.,$remainingPorts,$clS,$remainingCAPS,$segmentDurationMinutes,$expectedCLCalls"

		}
		else { #CR Plan CAPS is equal to or more than the needed CAPS, so just use CR
			$crS = [int][Math]::Ceiling($segmentPorts / $usedCrCAPS)
			$crRampUpTime = [timespan]::fromseconds($crS)
			("{0:hh\:mm\:ss}" -f $crRampUpTime)

			$expectedCRCallsMsg = ""
			if ($requiredSegmentCalls -gt 0) {
				$expectedCRCallsMsg = " out of the required $requiredSegmentCalls calls"
			}

			$crDescription = "CR Plan can handle both the needed CAPS and the port requirement for this segment. It should make $expectedSegmentCalls calls$expectedCRCallsMsg."

			$crBody = "{
			`"planId`": $crPlan,
			`"active`": false,
			`"channel`": `"Voice`",
			`"name`": `"$testDate Segment $segmentCount.0 $($AZ[$segmentCount-1]) | $segmentPorts Ports | $usedCrCAPS CAPS | $segmentDurationMinutes Minutes | CR`",
			`"description`": `"$crDescription`",
			`"concurrency`": $segmentPorts,
			`"maximumAps`": $usedCrCAPS,
			`"rampUpTime`": `"$crRampUpTime`",
			`"runTime`": `"$runTime`",
			`"requestedRunDate`": `"$runDate`",
			`"scheduledRunDate`": `"$runDate`",
			`"testCaseDistributionProfile`": `"$testCaseDistributionProfile`",
			`"planType`": `"Cruncher`",
			$testCaseJson
			}"

			#Create the Cruncher Campaign via the API
			$crReturn = createCampaign $crBody

			addCampaignToRunSheet "$segmentCount.0,$($AZ[$segmentCount-1]),CR Plan can handle both the needed CAPS and the port requirement for this segment.,$segmentPorts,$crS,$usedCrCAPS,$segmentDurationMinutes"
		}
	}
	else { #CR does not have enough ports on its own, thus we need to use both CR and CL

		#Calculate how many CAPS CR can support if we used every CR port
		$crCAPS = [math]::Round($crMaxPorts / 0.8 / $averageTestCaseMinuteDuration / 0.6 / 100, 2)

		#If that calculated CR amount is higher than CR plan, use CR plan's max because we're restricted and must
		if ($crCAPS -gt $crMaxCAPS) {
			$crCAPS = $crMaxCAPS
		}

		#We now check how many ports we can support using CR's max caps
		$crPorts = [int][Math]::Floor(0.8 * $averageTestCaseMinuteDuration * $crCAPS * 0.6 * 100)

		#If the calculated ports we can maintain is more than the max of CR, just use CR's max ports
		if ($crMaxPorts -lt $crPorts) {
			$crPorts = $crMaxPorts
		}

		logMessage "crCAPS: $crCAPS"
		logMessage "crPorts: $crPorts"

		$crS = [int][Math]::Ceiling($crPorts / $crCAPS)
		$crRampUpTime = [timespan]::fromseconds($crS)
		("{0:hh\:mm\:ss}" -f $crRampUpTime)

		$remainingPorts = $segmentPorts - $crPorts
		logMessage "remainingPorts $remainingPorts"
		$remainingCAPS = $usedCAPS - $crCAPS
		logMessage "remainingCAPS $remainingCAPS"
		if ($remainingCAPS -eq 0) {
			$remainingCAPS = 1
		}

		$clS = [int][Math]::Ceiling($remainingPorts / $remainingCAPS)
		$clRampUpTime = [timespan]::fromseconds($clS)
		("{0:hh\:mm\:ss}" -f $clRampUpTime)

		if ($clMaxPorts -lt $remainingPorts) {
			Throw "Missing CruncherLite Ports Error
			Based on your Cruncher plan's max CAPS, $remainingPorts ports were routed to Cruncher Lite
			Your Cruncher Lite plan does not have enough ports for these
			Please either increase your CR CAPS limit or increase your CL ports"
		}

		if ($clMaxCAPS -lt $remainingCAPS) {
			Throw "Missing CruncherLite CAPS Error
			Based on your Cruncher plan's max CAPS, $remainingPorts ports were routed to Cruncher Lite which requires $remainingCAPS CAPS
			Your Cruncher Lite plan does not have enough CAPS for these
			Please either increase your CR CAPS limit or increase your CL CAPS"
		}

		$expectedCRCalls = [Math]::Floor([decimal]($segmentDurationMinutes/$averageTestCaseMinuteDuration)) * $crPorts
		$expectedCRCallsMsg = ""
		if ($requiredSegmentCalls -gt 0) {
			$expectedCRCallsMsg = " out of the required $requiredSegmentCalls calls"
		}

		$crBody = "{
		`"planId`": $crPlan,
		`"active`": false,
		`"channel`": `"Voice`",
		`"name`": `"$testDate Segment $segmentCount.1 $($AZ[$segmentCount-1]) | $crPorts Ports | $crCAPS CAPS | $segmentDurationMinutes Minutes | CR`",
		`"description`": `"CR doesn't have enough ports alone thus this campaign must be run with its sibling CL. This CR campaign should make $expectedCRCalls calls$expectedCRCallsMsg.`",
		`"concurrency`": $crPorts,
		`"maximumAps`": $crCAPS,
		`"rampUpTime`": `"$crRampUpTime`",
		`"runTime`": `"$runTime`",
		`"requestedRunDate`": `"$runDate`",
		`"scheduledRunDate`": `"$runDate`",
		`"testCaseDistributionProfile`": `"$testCaseDistributionProfile`",
		`"planType`": `"Cruncher`",
		$testCaseJson
		}"

		logMessage "crBody $crBody"

		#Create the Cruncher Campaign via the API
		$crReturn = createCampaign $crBody

		addCampaignToRunSheet "$segmentCount.1,$($AZ[$segmentCount-1]),CR doesn't have enough ports alone thus this campaign must be run with its sibling CL.,$crPorts,$crS,$newCRCaps,$segmentDurationMinutes,$expectedCRCalls"

		$expectedCLCalls = [Math]::Floor([decimal]($segmentDurationMinutes/$averageTestCaseMinuteDuration)) * $remainingPorts
		$expectedCLCallsMsg = ""
		if ($requiredSegmentCalls -gt 0) {
			$expectedCLCallsMsg = " out of the required $requiredSegmentCalls calls"
		}

		$clBody = "{
		`"planId`": $clPlan,
		`"active`": false,
		`"channel`": `"Voice`",
		`"name`": `"$testDate Segment $segmentCount.2 $($AZ[$segmentCount-1]) | $remainingPorts Ports | $remainingCAPS CAPS | $segmentDurationMinutes Minutes | CL`",
		`"description`": `"The sibling CR campaign can't carry alone all the ports because of the plan limitation. This CL campaign picks up the slack. It should make $expectedCLCalls calls$expectedCLCallsMsg.`",
		`"concurrency`": $remainingPorts,
		`"maximumAps`": $remainingCAPS,
		`"rampUpTime`": `"$clRampUpTime`",
		`"runTime`": `"$runTime`",
		`"requestedRunDate`": `"$runDate`",
		`"scheduledRunDate`": `"$runDate`",
		`"testCaseDistributionProfile`": `"$testCaseDistributionProfile`",
		`"planType`": `"CruncherLite`",
		$testCaseJson
		}"

		#Create the Cruncher Lite Campaign via the API
		$clReturn = createCampaign $clBody

		addCampaignToRunSheet "$segmentCount.2,$($AZ[$segmentCount-1]),The sibling CR campaign can't carry alone all the ports because of the plan limitation. This CL campaign picks up the slack.,$remainingPorts,$clS,$remainingCAPS,$segmentDurationMinutes,$expectedCLCalls"

	}

	logMessage "Segment created"

}
elseif ($loadIncrement -eq "Stepped") {
	<#
	The goal of a stepped test is to increment evenly up to the full load.
	You will want to spent 15 minutes at each step to watch calls come out, checking for errors.
	Once fully ramped, you will want to stay there for at least 1/4 of the time. 4 hour test = 1 hour at full load
	Numerous recalculations are made to account for the stepping process.
	The number of steps to take is always higher is better, but there is a mathematical limit.
	We take into account the duration of the test and your ramp up times to determine what
	are the highest number of steps we can mathematically take.
	#>

	$fullyRampedSitTimeMinutes = $segmentDurationMinutes * .25
	$steppingTimeMinutes = $segmentDurationMinutes - $fullyRampedSitTimeMinutes
	$minutesSittingAtEachStage = 15

	$maximumSteps = [int][Math]::Floor($steppingTimeMinutes / $minutesSittingAtEachStage)
	logMessage "maximumSteps $maximumSteps"

	$usedSteps = -1
	$reservedStepDurationMinutes = -1

	#Loop from theoretical max steps trying to find the mathematical max steps
	for ($steps = $maximumSteps; $steps -ge 1; $steps--) {
		logMessage "steps $steps"
		$stepPorts = [math]::Round($segmentPorts / $steps, 2)
		$stepCAPS = [math]::Round($usedCAPS / $steps, 2)
		$stepRampUpSeconds = [int][Math]::Ceiling($stepPorts / $stepCAPS)

		$stepDuration = $minutesSittingAtEachStage * 60 + $stepRampUpSeconds
		logMessage "stepRampUpSeconds $stepRampUpSeconds"

		#we subtract minutesSittingAtEachStage because we don't count sitting on the last stage. That's part of fullyRampedSitTimeMinutes
		if ($stepDuration * $steps - $minutesSittingAtEachStage * 60 -lt $steppingTimeMinutes * 60 -and $stepCAPS * $steps -le $usedCAPS -and $stepPorts -ge 1) {
			$usedSteps = $steps
			$reservedStepDurationMinutes = [int][Math]::Floor($stepDuration / 60)
			break
		}
	}

	logMessage "reservedStepDurationMinutes $reservedStepDurationMinutes"
	logMessage "usedSteps $usedSteps"

	if ($usedSteps -eq 1) {
		Throw "Unsteppable Error
		Based on your ports and CAPS rate, you will not be able to ramp up in time.
		Please increase your test segment duration or CAPS rate, or decrease your ports"
	}

	if ($requiredSegmentCalls -gt 0) {
		#For each step, calculate how many calls we expect and add it to the counter. The step duration is equal to the total time - (reservedTime * stepIndex)
		$expectedSegmentCalls = 0;
		for ($steps = 0; $steps -lt $usedSteps; $steps++) {
			$expectedSegmentCalls = $expectedSegmentCalls + [Math]::Floor([decimal](($segmentDurationMinutes - $reservedStepDurationMinutes * $steps)/$averageTestCaseMinuteDuration)) * [int][Math]::Floor($segmentPorts / $usedSteps)
		}

		if ($expectedSegmentCalls -lt $requiredSegmentCalls) { #Calculated Calls Per Segment is less than customer's requested
			$extraMsg = ""
			if ($segmentPorts -gt 0) {
				$extraMsg = "but cannot go below $minimumAvgTestCaseMinuteDuration minutes to maintain your port requirement"
			}

			Throw "Too Few Calls Per Stepped Segment Error
			Even at max ports for the entire segment, you will only achieve
			$expectedSegmentCalls calls
			which is less than the required
			$requiredSegmentCalls calls
			Consider decreasing your average test case duration or increasing your ports
			With this average test case duration, you would need at least $minimumSegmentPorts ports
			With this port count, your average test case duration cannot exceed $minimumSegCallsAvgTestCaseDuration minutes to achieve your total segment call requirement
			$extraMsg"
		}
		elseif ($expectedSegmentCalls -gt $requiredSegmentCalls * 1.1 -and $expectedSegmentCalls -gt $requiredSegmentCalls + 500) { #Calculated Calls Per Segment greatly exceeds the customer's requested
			Throw "Too Many Calls Per Stepped Segment Error
			With your setup, you will achieve
			an estimated $expectedSegmentCalls calls
			which is far more than the required
			$requiredSegmentCalls calls.
			Consider increasing your average test case duration or decreasing your ports
			With this average test case duration, you would need at most $minimumSegmentPorts ports
			With this port count, your average test case duration cannot go below $minimumSegCallsAvgTestCaseDuration minutes to achieve your total segment call requirement"
		}
	}

	$usedStepPorts = [int][Math]::Floor($segmentPorts / $usedSteps)
	$usedStepCAPS = [math]::Round($usedCAPS / $usedSteps, 2)
	$usedStepRampUpSeconds = [int][Math]::Ceiling($usedStepPorts / $usedStepCAPS)
	$usedStepRampUpTime = [timespan]::fromseconds($usedStepRampUpSeconds)
	("{0:hh\:mm\:ss}" -f $usedStepRampUpTime)

	$remainingPorts = $segmentPorts
	$remainingCAPS = $usedCAPS
	$remainingCRPorts = $crMaxPorts
	$remainingCRCaps = $crMaxCAPS
	$remainingCLPorts = $clMaxPorts
	$remainingCLCaps = $clMaxCAPS

	#Do a dry run of the campaign creation loop to check for any errors first
	for ($curStep = $0; $curStep -lt $usedSteps; $curStep++) {
		$remainingStepPorts = $usedStepPorts
		$remainingStepCAPS = $usedStepCAPS

		#this is the last step make sure we grab the actual remaining ports and CAPS
		if ($curStep + 1 -eq $usedSteps) {
			$remainingStepPorts = $remainingPorts
			$remainingStepCAPS = $remainingCAPS
		}

		#CR has both ports and CAPS available, meaning we can still use it
		if ($remainingCRPorts -gt 0 -and $remainingCRCAPS -gt 0) {
			$usedCRPorts = $remainingCRPorts
			if ($usedCRPorts -gt $remainingStepPorts) {
				$usedCRPorts = $remainingStepPorts
			}
			$usedCRPortPctOfRemainingStepPorts = [math]::Round($usedCRPorts / $remainingStepPorts, 2)
			$usedCRCaps = [math]::Round($remainingStepCAPS * $usedCRPortPctOfRemainingStepPorts, 2)

			logMessage $usedCRCaps

			if ($usedCRCaps -gt $remainingCRCaps) {
				$usedCRCaps = $remainingCRCaps

				$usedCRCapsPctOfRemainingStepCaps = [math]::Round($usedCRCaps / $remainingStepCAPS, 2)
				$usedCRPorts = [int][Math]::Floor($remainingStepPorts * $usedCRCapsPctOfRemainingStepCaps)
				if ($usedCRPorts -eq 0) {
					$usedCRPorts = 1
				}
			}

			$remainingPorts = $remainingPorts - $usedCRPorts
			$remainingCaps = $remainingCaps - $usedCRCaps
			$remainingStepPorts = $remainingStepPorts - $usedCRPorts
			$remainingStepCaps = $remainingStepCaps - $usedCRCaps
			$remainingCRPorts = $remainingCRPorts - $usedCRPorts
			$remainingCRCaps = $remainingCRCaps - $usedCRCaps


		}

		#there are still ports remaining for this step, let's have CL pick up the slack
		if ($remainingStepPorts -gt 0 -and $remainingCLPorts -gt 0 -and $remainingCLCAPS -gt 0) {
			$usedCLPorts = $remainingStepPorts
			$usedCLCaps = $remainingStepCAPS

			#The amount of CL that we're need for this step exceeds what we have available.
			if ($usedCLPorts -gt $remainingCLPorts -and $usedCLCaps -gt $remainingCLCaps) {
				Throw "Insufficient CL Ports or CAPS error
				Based on your ports and CAPS for your CL plan, you do not have enough ports or CAPS for this step. Details are below:
				Required remaining CL ports: $remainingStepPorts
				Available remaining CL ports: $remainingCLPorts
				Required remaining CL CAPS: $remainingStepCaps
				Available remaining CL CAPS: $remainingCLCaps
				Please either increase your CL plan, your CR plan or change your test setup."
			}


			$remainingPorts = $remainingPorts - $usedCLPorts
			$remainingCaps = $remainingCaps - $usedCLCaps
			$remainingStepPorts = $remainingStepPorts - $usedCLPorts
			$remainingStepCaps = $remainingStepCaps - $usedCLCaps
			$remainingCLPorts = $remainingCLPorts - $usedCLPorts
			$remainingCLCaps = $remainingCLCaps - $usedCLCaps
		}

		if ($remainingStepPorts -gt 0) {
			Throw "Insufficient Ports or CAPS error
			Based on your ports and CAPS for your CR and CL plans, you are not able to stairstep efficiently.
			This error is most likely caused by an insufficient CAPS rate for your plans.
			Please try increasing your plans' Max CAPS, increasing your test segment duration, or decreasing your ports"
		}
	}

	$remainingPorts = $segmentPorts
	$remainingCAPS = $usedCAPS
	$remainingCRPorts = $crMaxPorts
	$remainingCRCaps = $crMaxCAPS
	$remainingCLPorts = $clMaxPorts
	$remainingCLCaps = $clMaxCAPS

	$runCounter = 1
	$letterCounter = $segmentCount-1

	#Actual loop to create campaigns
	for ($curStep = $0; $curStep -lt $usedSteps; $curStep++) {
		$remainingStepPorts = $usedStepPorts
		$remainingStepCAPS = $usedStepCAPS

		#this is the last step make sure we grab the actual remaining ports and CAPS
		if ($curStep + 1 -eq $usedSteps) {
			$remainingStepPorts = $remainingPorts
			$remainingStepCAPS = $remainingCAPS
		}

		#CR has both ports and CAPS available, meaning we can still use it
		if ($remainingCRPorts -gt 0 -and $remainingCRCAPS -gt 0) {
			$usedCRPorts = $remainingCRPorts
			if ($usedCRPorts -gt $remainingStepPorts) {
				$usedCRPorts = $remainingStepPorts
			}
			$usedCRPortPctOfRemainingStepPorts = [math]::Round($usedCRPorts / $remainingStepPorts, 2)
			$usedCRCaps = [math]::Round($remainingStepCAPS * $usedCRPortPctOfRemainingStepPorts, 2)

			logMessage $usedCRCaps

			if ($usedCRCaps -gt $remainingCRCaps) {
				$usedCRCaps = $remainingCRCaps

				$usedCRCapsPctOfRemainingStepCaps = [math]::Round($usedCRCaps / $remainingStepCAPS, 2)
				$usedCRPorts = [int][Math]::Floor($remainingStepPorts * $usedCRCapsPctOfRemainingStepCaps)
				if ($usedCRPorts -eq 0) {
					$usedCRPorts = 1
				}
			}

			$usedCRStepRampUpSeconds = [int][Math]::Ceiling($usedCRPorts / $usedCRCaps)
			$usedCRStepRampUpTime = [timespan]::fromseconds($usedCRStepRampUpSeconds)
			("{0:hh\:mm\:ss}" -f $usedCRStepRampUpTime)

			$expectedCRCalls = [Math]::Floor([decimal]($segmentDurationMinutes/$averageTestCaseMinuteDuration)) * $usedCRPorts
			$expectedCRCallsMsg = ""
			if ($requiredSegmentCalls -gt 0) {
				$expectedCRCallsMsg = " out of the required $requiredSegmentCalls calls"
			}

			$crBody = "{
			`"planId`": $crPlan,
			`"active`": false,
			`"channel`": `"Voice`",
			`"name`": `"$testDate Segment $segmentCount.$(($runCounter).ToString().PadLeft(3,'0')) $($AZ[$letterCounter]) | $usedCRPorts Ports | $usedCRCaps CAPS | $segmentDurationMinutes Minutes | CR`",
			`"description`": `"This CR campaign should make $expectedCRCalls calls$expectedCRCallsMsg.`",
			`"concurrency`": $usedCRPorts,
			`"maximumAps`": $usedCRCaps,
			`"rampUpTime`": `"$usedCRStepRampUpTime`",
			`"runTime`": `"$runTime`",
			`"requestedRunDate`": `"$runDate`",
			`"scheduledRunDate`": `"$runDate`",
			`"testCaseDistributionProfile`": `"$testCaseDistributionProfile`",
			`"planType`": `"Cruncher`",
			$testCaseJson
			}"

			logMessage "crBody $crBody"

			#Create the Cruncher Campaign via the API
			$crReturn = createCampaign $crBody

			addCampaignToRunSheet "$segmentCount.$(($runCounter).ToString().PadLeft(3,'0')),$($AZ[$letterCounter]),CR Campaign for test segment $segmentCount,$crPorts,$usedCRStepRampUpSeconds,$usedCRCaps,$segmentDurationMinutes,$expectedCRCalls"

			$remainingPorts = $remainingPorts - $usedCRPorts
			$remainingCaps = $remainingCaps - $usedCRCaps
			$remainingStepPorts = $remainingStepPorts - $usedCRPorts
			$remainingStepCaps = $remainingStepCaps - $usedCRCaps
			$remainingCRPorts = $remainingCRPorts - $usedCRPorts
			$remainingCRCaps = $remainingCRCaps - $usedCRCaps

			$runCounter = $runCounter + 1
		}

		#there are still ports remaining for this step, let's have CL pick up the slack
		if ($remainingStepPorts -gt 0 -and $remainingCLPorts -gt 0 -and $remainingCLCAPS -gt 0) {
			$usedCLPorts = $remainingStepPorts
			$usedCLCaps = $remainingStepCAPS


			$usedCLStepRampUpSeconds = [int][Math]::Ceiling($usedCLPorts / $usedCLCaps)
			$usedCLStepRampUpTime = [timespan]::fromseconds($usedCLStepRampUpSeconds)
			("{0:hh\:mm\:ss}" -f $usedCLStepRampUpTime)

			$expectedCLCalls = [Math]::Floor([decimal]($segmentDurationMinutes/$averageTestCaseMinuteDuration)) * $usedCLPorts
			$expectedCLCallsMsg = ""
			if ($requiredSegmentCalls -gt 0) {
				$expectedCLCallsMsg = " out of the required $requiredSegmentCalls calls"
			}

			$clBody = "{
			`"planId`": $clPlan,
			`"active`": false,
			`"channel`": `"Voice`",
			`"name`": `"$testDate Segment $segmentCount.$(($runCounter).ToString().PadLeft(3,'0')) $($AZ[$letterCounter]) | $usedCLPorts Ports | $usedCLCaps CAPS | $segmentDurationMinutes Minutes | CL`",
			`"description`": `"This CL campaign should make $expectedCLCalls calls$expectedCLCallsMsg.`",
			`"concurrency`": $usedCLPorts,
			`"maximumAps`": $usedCLCaps,
			`"rampUpTime`": `"$usedCLStepRampUpTime`",
			`"runTime`": `"$runTime`",
			`"requestedRunDate`": `"$runDate`",
			`"scheduledRunDate`": `"$runDate`",
			`"testCaseDistributionProfile`": `"$testCaseDistributionProfile`",
			`"planType`": `"CruncherLite`",
			$testCaseJson
			}"

			logMessage "clBody $clBody"

			#Create the Cruncher Lite Campaign via the API
			$clReturn = createCampaign $clBody

			addCampaignToRunSheet "$segmentCount.$(($runCounter).ToString().PadLeft(3,'0')),$($AZ[$letterCounter]),CL Campaign for test segment $segmentCount,$clPorts,$usedCLStepRampUpSeconds,$usedCLCaps,$segmentDurationMinutes,$expectedCLCalls"


			$remainingPorts = $remainingPorts - $usedCLPorts
			$remainingCaps = $remainingCaps - $usedCLCaps
			$remainingStepPorts = $remainingStepPorts - $usedCLPorts
			$remainingStepCaps = $remainingStepCaps - $usedCLCaps
			$remainingCLPorts = $remainingCLPorts - $usedCLPorts
			$remainingCLCaps = $remainingCLCaps - $usedCLCaps

			$runCounter = $runCounter + 1
		}

		$letterCounter = $letterCounter + 1

	}
}