package com.cyara.apisamples;

import java.util.ArrayList;
import java.util.Collections;

import javax.json.JsonArray;
import javax.json.JsonObject;

/**
 * For a given timeframe, will grab up to 1000 Pulse calls and
 * produce results showing you every unique prompt, and
 * all the different confidence score results for each prompt.
 * This allows you to adjust confidence scores accordingly, great
 * when building out your Pulse library. This program works best
 * when all your prompts are blockified without duplicates.
 * <p>
 * This is not an official Cyara product and is not officially supported.
 * It is used for demonstration and training purposes. For assistance,
 * contact chris.ryan@cyara.com
 * 
 * @author Chris Ryan
 * @version 1.1
 */
public class AdjustConfidences {
	public static void main(String[] argv) {
		ApiFunctionalities connInfo = new ApiFunctionalities();

		JsonObject apiObject = connInfo.getTestResultsFeed("pulse", "fromDate","2020-04-01T12:00:00-05:00");
		JsonArray callsArr = apiObject.getJsonArray("results");

		ArrayList<String> expectToHears = new ArrayList<>();
		ArrayList<String> confidenceScores = new ArrayList<>();
		ArrayList<String> outcomes = new ArrayList<>();

		/*
		 * Loop through every Pulse call returned by
		 * the API and capture confidence scores and
		 * results for every unique prompt.
		 */
		for (int x = 0; x < callsArr.size(); x++) {
			JsonObject call = callsArr.getJsonObject(x);
			JsonObject testStepResultList = call.getJsonObject("testStepResultList");
			JsonArray callStepResult = testStepResultList.getJsonArray("callStepResult");

			//Loop thru all the steps for this call
			for (int stepCnt = 0; stepCnt < callStepResult.size(); stepCnt++) {
				JsonObject stepDetails = callStepResult.getJsonObject(stepCnt);
				JsonObject step = stepDetails.getJsonObject("step");
				JsonObject expect = step.getJsonObject("expect");
				String ethText = expect.getString("text");

				JsonObject stepResult = stepDetails.getJsonObject("stepResult");
				String stepResultDetail = stepResult.getString("detail");

				JsonObject confidence = step.getJsonObject("confidence");
				String confidenceMajorMinor = confidence.getInt("major") + "/" + confidence.getInt("minor");

				//Check if this step has the result detail for poor confidence
				if (stepResultDetail.contains("is less than the required")) {
					confidenceMajorMinor = confidenceMajorMinor + " FAILED";
				}
				else {
					confidenceMajorMinor = confidenceMajorMinor + " GOOD";
				}

				//If this step is a prompt, process it
				if (ethText.length() > 0) {
					String confidenceScore = "0";
					if (stepResultDetail.contains("onfidence:")) {
						confidenceScore = stepResultDetail.substring(stepResultDetail.indexOf("onfidence: ")+11, stepResultDetail.indexOf("%"));
					}

					//If this is is a known text, add it to previous index
					if (expectToHears.contains(ethText)) {
						int textLocation = expectToHears.indexOf(ethText);
						confidenceScores.set(textLocation, confidenceScores.get(textLocation) + confidenceScore + "~");

						if (confidenceMajorMinor.contains("FAILED")) {
							outcomes.set(textLocation, confidenceMajorMinor);
						}
					}
					else { //If this is an unknown text, add it for the first time
						expectToHears.add(ethText);
						confidenceScores.add(confidenceScore + "~");
						outcomes.add(confidenceMajorMinor);
					}
				}
			}
		}


		/*
		 * Loop thru all the different prompts found
		 * and print out their overall worst result
		 * and metrics of their confidence scores
		 */
		for (int x = 0; x < expectToHears.size(); x++) {
			System.out.println(outcomes.get(x));
			System.out.println(expectToHears.get(x));
			String[] confArr = confidenceScores.get(x).split("~");
			ArrayList<Double> confList = new ArrayList<>();

			//Loop thru all the confidence scores and add to arraylist as Double
			for (int conf = 0; conf < confArr.length; conf++) {
				try {
					confList.add(Double.parseDouble(confArr[conf]));
				}
				catch (NumberFormatException ex) {

				}
			}

			String outcome = outcomes.get(x);

			//Store this prompt's minor and major confidence scores
			int majorScore = Integer.parseInt(outcome.substring(0, outcome.indexOf("/")));
			int minorScore = Integer.parseInt(outcome.substring(outcome.indexOf("/")+1, outcome.indexOf(" ")));

			Collections.sort(confList);

			//Print the highest and lowest confidence score result, and step usages
			System.out.println("Lowest: " + confList.get(0));
			System.out.println("Highest " + confList.get(confList.size()-1));
			System.out.println("Usages: " + confList.size());


			//Loop to count up all minor and major confidence breaches
			int minorBreaches = 0;
			int majorBreaches = 0;
			for (int n = 0; n < confList.size(); n++) {
				double confidenceScore = confList.get(n);
				if (confidenceScore <= majorScore) {
					majorBreaches++;
				}
				else if (confidenceScore <= minorScore) {
					minorBreaches++;
				}
			}

			System.out.println("Minor Breaches: " + minorBreaches);
			System.out.println("Major Breaches: " + majorBreaches);

			//Print out all the confidence score results for this prompt
			for (int n = 0; n < confList.size(); n++) {
				System.out.print(confList.get(n));
				if (n < confList.size()-1) {
					System.out.print(" + ");
				}
			}

			System.out.println("\n");

		}

	}	
}
