package com.cyara.apisamples;

import javax.json.JsonArray;
import javax.json.JsonObject;

import com.cyara.utilities.Utils;
import com.cyara.utilities.UtilsMath;

/**
 * This sample takes in a test case ID and creates a
 * debug duplicate, ANI spoofing it with a random,
 * unique ANI. The test case is then run, and 60 seconds
 * later the result is outputted. Great to automatically
 * execute a call and search your backend for the unique ANI
 * if it fails.
 * <p>
 * This is not an official Cyara product and is not officially supported.
 * It is used for demonstration and training purposes. For assistance,
 * contact chris.ryan@cyara.com
 * 
 * @author Chris Ryan
 * @version 1.3
 */
public class AniDebugTestCase {
	public static void main(String[] argv) {
		ApiFunctionalities connInfo = new ApiFunctionalities();

		//Change to your test case ID
		int failedTestCaseId = -1;

		String failedTestCaseXml = connInfo.exportTestCase(failedTestCaseId, "IncludeBlocks");

		//Create the last 8 digits randomly for the ANI
		int aniSpoof = UtilsMath.getRandomNumber(30000000, 39999999);

		//Get the current destination
		String phoneNo = failedTestCaseXml.substring(failedTestCaseXml.indexOf("<PhoneNo>")+9, failedTestCaseXml.indexOf("</PhoneNo>"));

		//Remove any current ANI spoofing
		if (phoneNo.contains(";From:")) {
			phoneNo = phoneNo.substring(0, phoneNo.indexOf(";"));
		}

		//Update destination with new ANI spoof
		phoneNo = phoneNo + ";From:181" + aniSpoof;

		//Update test case XML with new name, folder, and ANI spoof
		failedTestCaseXml = failedTestCaseXml.replace("</TestCaseName>", "_" + System.currentTimeMillis() + "</TestCaseName>")
				.replaceAll("<FolderPath>.*?<\\/FolderPath>", "<FolderPath>Debug<\\/FolderPath>")
				.replaceAll("<PhoneNo>.*?<\\/PhoneNo>", "<PhoneNo>" + phoneNo + "<\\/PhoneNo>");

		/*
		 * Import test case XML.
		 * A new test case is created because we
		 * changed the test case name and folder.
		 * (Either one of those being changed creates
		 * a new test case.)
		 */
		JsonObject importInfo = connInfo.importTestCaseFromString(failedTestCaseXml);

		JsonArray results = importInfo.getJsonArray("results");

		JsonObject importOutcome = results.getJsonObject(0);

		//Get test case of newly created test case
		int newTestCaseId = importOutcome.getInt("id");

		//Run the new test case
		JsonObject newTestCaseExecution = connInfo.executeTestCase(newTestCaseId);

		System.out.println(newTestCaseExecution.toString());

		String testRunTicketId = newTestCaseExecution.getString("testRunTicketId");

		Utils.sleepMode(60000);

		//Get the test case run result
		JsonObject testCaseResult = connInfo.getTestCaseResultByTicketId(testRunTicketId);
		System.out.println(testCaseResult.toString());

		JsonObject testCaseInfo = testCaseResult.getJsonObject("testCase");

		/*
		 * We now have the result json. The result
		 * details and run information are grabbed.
		 */

		int testCaseId = testCaseInfo.getInt("testCaseId");

		String testCaseName = testCaseInfo.getString("name");

		String folderPath = "\\";
		if (!testCaseInfo.isNull("folderPath")) {
			folderPath = testCaseInfo.getString("folderPath");
		}

		int testResultId = testCaseResult.getInt("testResultId");

		JsonObject testResult = testCaseResult.getJsonObject("testResult");
		String result = testResult.getString("result");

		//If the call result was anything but success, print the details
		if (!result.equals("Success")) {
			String detail = testResult.getString("detail").trim();

			System.out.print("This test case was " + result + " because of " + detail + ": " + folderPath + " | " + testCaseName + " | " + testCaseId + " | " + testResultId);
		}
	}

}
