package com.cyara.apisamples;

import java.time.Instant;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;

import javax.json.JsonArray;
import javax.json.JsonObject;

import com.cyara.utilities.Utils;

/**
 * This program will grab every validation that occurred in a
 * campaign for the last X months. It will export to a csv the
 * duration of each call in minutes, its result, and other misc info.
 * This data can then be used to calculate usage, errors found, time saved, etc.
 * <p>
 * This is not an official Cyara product and is not officially supported.
 * It is used for demonstration and training purposes. For assistance,
 * contact chris.ryan@cyara.com
 * 
 * @author Chris Ryan
 * @version 1.2
 */
public class CyaraSavings { 

	public static void main (String[] argv) {
		ApiFunctionalities connInfo = new ApiFunctionalities();

		long runTime = Instant.now().getEpochSecond();

		//Log the headers
		Utils.logMessage("Campaign Type,Campaign Name,Campaign ID,Run ID,Test Result ID,Test Case ID,Called Number,Test Case Result,Test Case Run Date,Call Duration", runTime + "_Savings.csv", true);

		/*
		 * The first int in the method call represents
		 * the number of months to grab. Change as needed.
		 */
		//calculateLastXMonths(1, "pulse", runTime, connInfo);
		calculateLastXMonths(1, "replay", runTime, connInfo);
		calculateLastXMonths(1, "velocity", runTime, connInfo);
	}

	/**
	 * The brain behind the program that pulls and logs all
	 * the data you need for calculating savings
	 * 
	 * @param months The number of months to retrieve calls from
	 * @param planType The plan we're retrieving calls for
	 * @param runTime Timestamp used for logging purposes
	 * @param connInfo The controller to the API credentials and methods
	 */
	private static void calculateLastXMonths(int months, String planType, long runTime, ApiFunctionalities connInfo) {
		//Create a specific date and time from scratch, subtract X days from it, then format it to yyyy-MM-ddThh:mm:ssZ
		String previousTime = DateTimeFormatter.ISO_INSTANT.format(Instant.now().minus(months * 30, ChronoUnit.DAYS));

		boolean morePages = true;
		String nextPage = "";

		JsonObject apiObject = connInfo.getTestResultsFeed(planType, "fromDate", previousTime);

		while (morePages) {
			if (apiObject.containsKey("selfLink")) {
				String selfLink = apiObject.getString("selfLink");
				String nextLink = apiObject.getString("nextLink");
				if (selfLink.equals(nextLink)) {
					morePages = false;
				}
				else {
					nextPage = apiObject.getString("nextLink");
				}

				JsonArray resultsArr = apiObject.getJsonArray("results");
				for (int resultIndex = 0; resultIndex < resultsArr.size(); resultIndex++) {
					//Get specific call
					JsonObject call = resultsArr.getJsonObject(resultIndex);

					JsonObject campaign = call.getJsonObject("campaign");
					int campaignId = campaign.getInt("campaignId");
					String campaignName = campaign.getString("campaignName");
					int runId = campaign.getInt("runId");


					int testResultId = call.getInt("testResultId");


					JsonObject testCase = call.getJsonObject("testCase");
					int testCaseId = testCase.getInt("testCaseId");
					String calledNumber = testCase.getString("calledNumber");

					JsonObject testResult = call.getJsonObject("testResult");
					String result = testResult.getString("result");


					String runDate = call.getString("runDate");


					String duration = call.getString("duration");

					//Assemble all the call's information, then log it
					String outputString = planType + "," + campaignName + "," + campaignId + "," + runId + "," + testResultId + "," + testCaseId + "," + calledNumber + "," + result + "," + runDate + "," + duration; 
					Utils.logMessage(outputString, runTime + "_Savings.csv", true);
				}
			}
			else { //nextPage doesn't even exist, something is totally wrong
				morePages = false;
			}

			if (morePages) { //After processing that call, do we still have pages? Then make the next call here
				apiObject = connInfo.convertAPIOutputToJson(connInfo.makeApiGet(nextPage));
			}
		}

	}

}
