package com.cyara.apisamples;

import java.util.ArrayList;

import javax.json.JsonArray;
import javax.json.JsonObject;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * This program will run through all your blocks
 * and print out all the duplicate blocks, showing
 * the duplicate and the original.
 * <p>
 * Works best if your blocks follow the single step best practice.
 * <p>
 * If desired, also has the capability to replace duplicated blocks
 * with the original and delete the duplicate. THIS CANNOT BE UNDONE!
 * <p>
 * This is not an official Cyara product and is not officially supported.
 * It is used for demonstration and training purposes. For assistance,
 * contact chris.ryan@cyara.com
 * 
 * @author Chris Ryan
 * @version 1.1
 */
public class FindDuplicateBlocks {
	public static void main(String[] argv) {
		ApiFunctionalities connInfo = new ApiFunctionalities();

		ArrayList<Integer> allBlockIDs = new ArrayList<>();

		boolean morePages = true;
		String nextPage = "";

		JsonObject allBlocksObject = connInfo.listBlocks();

		//Loop through all the pages of listed blocks, storing all block IDs
		while (morePages) {
			if (allBlocksObject.containsKey("nextPage")) {
				if (allBlocksObject.isNull("nextPage")) {
					morePages = false;
				}
				else {
					nextPage = allBlocksObject.getString("nextPage");
				}

				JsonArray blockArr = allBlocksObject.getJsonArray("results");
				for (int blockIndex = 0; blockIndex < blockArr.size(); blockIndex++) {
					JsonObject block = blockArr.getJsonObject(blockIndex);
					int blockID = block.getInt("blockId");
					allBlockIDs.add(blockID);
				}

			}
			else { //nextPage doesn't even exist, something is totally wrong
				morePages = false;
			}

			if (morePages) { //After processing that call, do we still have pages? Then make the next call here
				allBlocksObject = connInfo.convertAPIOutputToJson(connInfo.makeApiGet(nextPage));
			}
		}

		ArrayList<String> knownRawBlockETHs = new ArrayList<>();
		ArrayList<String> knownCleanBlockETHs = new ArrayList<>();
		ArrayList<Integer> knownBlockIDs = new ArrayList<>();

		//Main loop thru all the blocks, exporting them as XML and processing
		for (int blockIndex = 0; blockIndex < allBlockIDs.size(); blockIndex++) {
			System.out.println("Checking " + allBlockIDs.get(blockIndex));
			String rawXML = connInfo.exportBlock(allBlockIDs.get(blockIndex));

			//If this block has a prompt, we process it
			if (rawXML.contains("<ExpectedText>")) {

				String cleanETH = rawXML.substring(rawXML.indexOf("<ExpectedText>")+14, rawXML.indexOf("</ExpectedText>")).replaceAll("[^a-zA-Z 0-9]", "");
				cleanETH = cleanETH.replace("EndCallIfNoMatch", "");
				cleanETH = cleanETH.toLowerCase().trim();
				cleanETH = cleanETH.replace("  ", " ");

				//Flag used to check whether or not this block already exists
				boolean foundKnown = false;

				for (int searchIndex = 0; searchIndex < knownCleanBlockETHs.size(); searchIndex++) {
					String knownCleanETH = knownCleanBlockETHs.get(searchIndex);

					//Block is a dupe!
					if (cleanETH.equals(knownCleanETH)) {
						foundKnown = true;
						System.out.println("\n\nFound Match:");
						System.out.println(allBlockIDs.get(blockIndex) + " is a possible dupe of " + knownBlockIDs.get(searchIndex));
						System.out.println("Clean Text:");
						System.out.println(cleanETH);
						System.out.println(knownCleanETH);
						System.out.println("Raw Text:");

						String dupeBlockRaw = rawXML;
						String originalBlockRaw = connInfo.exportBlock(knownBlockIDs.get(searchIndex));

						//Boilerplate
						Document dupeBlockXml = connInfo.convertStringToDocument(dupeBlockRaw);
						dupeBlockXml.getDocumentElement().normalize();
						Document originalBlockXml = connInfo.convertStringToDocument(originalBlockRaw);
						originalBlockXml.getDocumentElement().normalize();

						//Collect dupe information to use
						Element dupeEle = dupeBlockXml.getDocumentElement();
						Element dupeBlockEle = (Element) dupeEle.getElementsByTagName("Block").item(0);
						System.out.println("~~~~~" + dupeBlockEle.getTextContent() + "");
						Node dupeBlockName = dupeBlockEle.getElementsByTagName("BlockName").item(0);
						System.out.println("dupeBlockName: " + dupeBlockName.getTextContent());
						String dupeBlockNameString = dupeBlockName.getTextContent();
						Node dupeFolderPath = dupeBlockEle.getElementsByTagName("FolderPath").item(0);
						System.out.println("dupeFolderPath: " + dupeFolderPath.getTextContent());
						String dupeFolderPathString = dupeFolderPath.getTextContent();

						//Collect original information to use
						Element originalEle = originalBlockXml.getDocumentElement();
						Element originalBlockEle = (Element) originalEle.getElementsByTagName("Block").item(0);
						Node originalBlockName = originalBlockEle.getElementsByTagName("BlockName").item(0);
						System.out.println("originalBlockName: " + originalBlockName.getTextContent());
						String originalBlockNameString = originalBlockName.getTextContent();
						Node originalFolderPath = originalBlockEle.getElementsByTagName("FolderPath").item(0);
						System.out.println("originalFolderPath: " + originalFolderPath.getTextContent());
						String originalFolderPathString = originalFolderPath.getTextContent();

						dupeBlockRaw = dupeBlockRaw.substring(dupeBlockRaw.indexOf("<Block>"), dupeBlockRaw.indexOf("</Blocks>")).trim();
						System.out.println("Dupe:");
						while (dupeBlockRaw.contains("  ")) {
							dupeBlockRaw = dupeBlockRaw.replace("	", " ").replace("  ", " ");
						}
						System.out.println(dupeBlockRaw);

						originalBlockRaw = originalBlockRaw.substring(originalBlockRaw.indexOf("<Block>"), originalBlockRaw.indexOf("</Blocks>")).trim().replaceAll("  ", " ");
						System.out.println("Original:");
						while (originalBlockRaw.contains("  ")) {
							originalBlockRaw = originalBlockRaw.replace("	", " ").replace("  ", " ");
						}
						System.out.println(originalBlockRaw);

						//Uncomment this if you want the program to automatically replace and delete the duplicated blocks

						/*
						JsonObject linkedTestCases = connInfo.getLinkedTestCases(allBlockIDs.get(blockIndex));
						JsonArray linkedArr = linkedTestCases.getJsonArray("results");
						for (int links = 0; links < linkedArr.size(); links++) {
							JsonObject linkDetails = linkedArr.getJsonObject(links);
							int testCaseId = linkDetails.getInt("testCaseId");

							String testCaseXml = connInfo.exportTestCase(testCaseId);

							System.out.println("Checking for " + "<BlockPath>" + dupeFolderPathString + "\\" + dupeBlockNameString + "</BlockPath>");

							testCaseXml = testCaseXml.replace("<BlockPath>" + dupeFolderPathString + "\\" + dupeBlockNameString + "</BlockPath>", "<BlockPath>" + originalFolderPathString + "\\" + originalBlockNameString + "</BlockPath>");

							while (testCaseXml.contains("  ")) {
								testCaseXml = testCaseXml.replace("	", " ").replace("  ", " ");
							}
							testCaseXml = testCaseXml.replace(dupeBlockRaw, originalBlockRaw);
							System.out.println("Cleaned!");
							System.out.println(testCaseXml);
							connInfo.importTestCaseFromString(testCaseXml);
							System.out.println(testCaseId);
							//System.exit(1);
						}
						 */

						break;
					}
				}

				//If we looped thru all known blocks and didn't find a dupe, store this
				if (!foundKnown) {
					knownRawBlockETHs.add(rawXML);
					knownCleanBlockETHs.add(cleanETH);
					knownBlockIDs.add(allBlockIDs.get(blockIndex));
				}
			}
		}

	}

}
