package com.cyara.apisamples;

import javax.json.JsonArray;
import javax.json.JsonObject;

/**
 * This basic program will grab up to 1000 calls from the
 * designated start time and print out a URL to
 * any call that had a latency problem (exceeded threshold).
 * <p>
 * Used as a great introduction to pulse processing and link
 * generation.
 * <p>
 * This is not an official Cyara product and is not officially supported.
 * It is used for demonstration and training purposes. For assistance,
 * contact chris.ryan@cyara.com
 * 
 * @author Chris Ryan
 * @version 1.0
 */
public class FindThresholdBreaches {
	public static void main(String[] argv) {
		ApiFunctionalities connInfo = new ApiFunctionalities();

		JsonObject apiObject = connInfo.getTestResultsFeed("pulse", "fromDate", "2020-02-14T15:25:56Z");
		JsonArray callsArr = apiObject.getJsonArray("results");

		//Loop thru all the grabbed pulse calls and process
		for (int x = 0; x < callsArr.size(); x++) {
			JsonObject call = callsArr.getJsonObject(x);
			int testResultId = call.getInt("testResultId");
			JsonObject testStepResultList = call.getJsonObject("testStepResultList");
			JsonArray callStepResult = testStepResultList.getJsonArray("callStepResult");

			/*
			 * Loop thru all the steps, checking for an exceeded threshold failure.
			 * We have to check every step because the overall call result failure
			 * only shows the first failure (usually, sometimes the worst). This means
			 * an exceeded threshold could come after a confidence failure and is
			 * only visible by processing the steps like we do below.
			 */
			for (int stepCnt = 0; stepCnt < callStepResult.size(); stepCnt++) {
				JsonObject stepDetails = callStepResult.getJsonObject(stepCnt);

				JsonObject stepResult = stepDetails.getJsonObject("stepResult");
				String stepResultDetail = stepResult.getString("detail");

				if (stepResultDetail.contains("exceeded")) {
					System.out.println(connInfo.getBaseUrl() + "cyarawebportal/" + connInfo.getAccountNum() + "/report/resultdetails?testResultId=" + testResultId + "&mediaType=Voice");
					break;
				}
			}
		}

	}

}
