package com.cyara.apisamples;

import java.util.ArrayList;
import java.util.Collections;

import javax.json.JsonArray;
import javax.json.JsonObject;

/**
 * This program prints out every current CX Model
 * on the account, and all the test cases generated
 * by that model. Used to show a count of test
 * cases per CX Model. Output is tilde separated.
 * <p>
 * This is not an official Cyara product and is not officially supported.
 * It is used for demonstration and training purposes. For assistance,
 * contact chris.ryan@cyara.com
 * 
 * @author Chris Ryan
 * @version 1.1
 */
public class ListTestCasesCXModels {
	public static void main(String[] argv) {
		ApiFunctionalities connInfo = new ApiFunctionalities();

		ArrayList<String> allTCs = new ArrayList<>();
		ArrayList<String> allModels = new ArrayList<>();
		ArrayList<Integer> allModelsTCCnt = new ArrayList<>();
		ArrayList<String> connectedTCs = new ArrayList<>();

		boolean morePages = true;
		String nextPage = "";
		JsonObject apiObject = connInfo.listTestCases();

		//Loop thru all the pages of test cases, storing their name and folder path
		while (morePages) {
			if (apiObject.containsKey("nextPage")) {
				if (apiObject.isNull("nextPage")) {
					morePages = false;
				}
				else {
					nextPage = apiObject.getString("nextPage");
				}

				JsonArray resultsArr = apiObject.getJsonArray("results");
				for (int resultIndex = 0; resultIndex < resultsArr.size(); resultIndex++) {
					//Get specific call
					JsonObject testCase = resultsArr.getJsonObject(resultIndex);

					String name = testCase.getString("name");

					String folderPath = testCase.getString("folderPath");

					if(!folderPath.contains("zzzCrawler") && !allTCs.contains(name.trim() + "~" + folderPath.trim())) {
						allTCs.add(name.trim() + "~" + folderPath.trim());
					}
				}
			}
			else { //nextPage doesn't even exist, something is totally wrong
				morePages = false;
			}

			if (morePages) { //After processing that call, do we still have pages? Then make the next call here
				apiObject = connInfo.convertAPIOutputToJson(connInfo.makeApiGet(nextPage));
			}
		}

		Collections.sort(allTCs);

		morePages = true;
		nextPage = "";
		apiObject = connInfo.listCXModels();

		//Loop thru all the pages of models, storing their name and adding index to lists
		while (morePages) {
			if (apiObject.containsKey("nextPage")) {
				if (apiObject.isNull("nextPage")) {
					morePages = false;
				}
				else {
					nextPage = apiObject.getString("nextPage");
				}

				JsonArray modelsArr = apiObject.getJsonArray("results");

				for (int modelIndex = 0; modelIndex < modelsArr.size(); modelIndex++) {
					JsonObject model = modelsArr.getJsonObject(modelIndex);
					String name = model.getString("name");
					allModels.add(name);
					allModelsTCCnt.add(0);
					connectedTCs.add("");
				}
			}
			else { //nextPage doesn't even exist, something is totally wrong
				morePages = false;
			}

			if (morePages) { //After processing that call, do we still have pages? Then make the next call here
				apiObject = connInfo.convertAPIOutputToJson(connInfo.makeApiGet(nextPage));
			}
		}

		//Loop thru all the test cases we grabbed to associate with their model
		for (int x = 0; x < allTCs.size(); x++) {
			String[] fullTCDetailArr = allTCs.get(x).split("~");
			String testCaseName = fullTCDetailArr[0].replace("\"", "");
			String testCaseSourceModel = fullTCDetailArr[1]; //This is currently the folder path, but becomes the model name...
			if (testCaseSourceModel.contains("\\")) {

				//...this is where it becomes the model name
				testCaseSourceModel = testCaseSourceModel.substring(0, testCaseSourceModel.indexOf("\\")).replace("\"", "");

				//Check if this model still exists, and then associate
				if (allModels.contains(testCaseSourceModel)) {
					int modelIndex = allModels.indexOf(testCaseSourceModel);

					//Increment number of test cases this model has generated
					int currentCount = allModelsTCCnt.get(modelIndex);
					currentCount++;
					allModelsTCCnt.set(modelIndex, currentCount);

					//Add this test case to list of test cases this model has generated
					String currentTCs = connectedTCs.get(modelIndex);
					connectedTCs.set(modelIndex, currentTCs + "~" + testCaseName);
				}
			}
		}

		//Print out the results
		System.out.println("TC Count~CX Model Name~Test Case Name");
		for (int x = 0; x < allModels.size(); x++) {
			if (allModelsTCCnt.get(x) > 1) {
				System.out.println(allModelsTCCnt.get(x) + "~" + allModels.get(x));

				String tcsRaw = connectedTCs.get(x);
				tcsRaw = tcsRaw.substring(1, tcsRaw.length());
				String[] linkedTCs = tcsRaw.split("~");
				for (int y = 0; y < linkedTCs.length; y++) {
					System.out.println("~~" + linkedTCs[y]);
				}
			}
		}

	}

}
