package com.cyara.apisamples;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;

import javax.json.Json;
import javax.json.JsonArray;
import javax.json.JsonObject;
import javax.json.JsonReader;
import javax.json.stream.JsonParsingException;
import javax.net.ssl.HttpsURLConnection;

import com.cyara.utilities.Utils;
import com.cyara.utilities.UtilsMath;
import com.cyara.utilities.UtilsTime;

/**
 * Training demo built for Xchange 2020 that uses Cyara's Pulse
 * API to collect call results and generate metrics into an HTML file.
 * <p>
 * This is not an official Cyara product and is not officially supported.
 * It is used for demonstration and training purposes. For assistance,
 * contact chris.ryan@cyara.com
 * 
 * @author Chris Ryan
 * @version 1.7
 */
public class PulseMetrics implements Comparable<PulseMetrics> {
	private int problemTestCaseId = -1;
	private String problemTestCaseName = "";
	private int problemTestCaseFailCount = 0;

	public static void main(String[] argv) {
		ApiFunctionalities connInfo = new ApiFunctionalities();
		SimpleDateFormat apiFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSSSSS'Z'");
		//SimpleDateFormat apiFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");

		String jsonFile = "StoredResults.json";
		String outputFile = "Metrics_" + System.currentTimeMillis() + ".html";

		//Check if the Json output file even exists
		File resultFile = new File(jsonFile);
		if (!resultFile.exists()) {
			try {
				resultFile.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		//Grab new pulse data and put in file
		while (true) {
			int lastTestResultId = getLastTestResultId(jsonFile);

			//Valid previously grabbed id found, increment by 1 for next ID
			lastTestResultId++;
			System.out.println("Pulling results from next ID " + lastTestResultId);

			JsonObject pulseJson = connInfo.getTestResultsFeed("pulse", "fromTestResultId", String.valueOf(lastTestResultId));

			String selfLink = pulseJson.getString("selfLink");
			String nextLink = pulseJson.getString("nextLink");

			if (nextLink.toLowerCase().equals(selfLink.toLowerCase())) {
				break;
			}

			Utils.logMessage(pulseJson.toString(), jsonFile, true);

		}

		//UPDATE ME AS DESIRED
		try {
			//getTimeframeStats(jsonFile, apiFormat, apiFormat.parse("2000-01-01T00:00:00.0000000Z"), new Date(), "All Time", baseUrl, accountNum);
			getTimeframeStats(jsonFile, apiFormat, apiFormat.parse("2020-04-01T00:00:00.0000000Z"), apiFormat.parse("2020-07-01T00:00:00.0000000Z"), "Calendar Q2 2020", connInfo.getBaseUrl(), connInfo.getAccountNum(), outputFile);
			//getTimeframeStats(jsonFile, apiFormat, getFirstDayOfThisQuarter(new Date()), getLastDayOfThisQuarter(new Date()), "This Quarter", baseUrl, accountNum, outputFile);
			//getTimeframeStats(jsonFile, apiFormat, getFirstDayOfLastQuarter(new Date()), getLastDayOfLastQuarter(new Date()), "Last Quarter", baseUrl, accountNum, outputFile);
			//getTimeframeStats(jsonFile, apiFormat, getFirstDayOfThisYear(new Date()), new Date(), "This Year", baseUrl, accountNum);

			ArrayList<String> comparedTimeframes = new ArrayList<String>();
			comparedTimeframes.add(apiFormat.parse("2020-04-03T00:00:00.0000000Z") + "," + apiFormat.parse("2020-04-04T00:00:00.0000000Z"));
			comparedTimeframes.add(apiFormat.parse("2020-05-01T00:00:00.0000000Z") + "," + apiFormat.parse("2020-05-02T00:00:00.0000000Z"));
			comparedTimeframes.add(apiFormat.parse("2020-06-05T00:00:00.0000000Z") + "," + apiFormat.parse("2020-06-06T00:00:00.0000000Z"));
			getComparisonStats(jsonFile, apiFormat, comparedTimeframes, outputFile);
		} catch (Exception e) {
			e.printStackTrace();
		}

	}


	/**
	 *  Take in one or more date ranges and build the necessary
	 *  Chart.JS structure for a scatter graph. Ultimately, the
	 *  method to generate said scatter graph is called from here.
	 *  <p>
	 *  The previously grabbed API data is accessed and response
	 *  times for Step 0 and Step 1 are stored. These reflect the
	 *  connection times and initiation times, respectively.
	 *  
	 *  @param jsonFile The source file of all the previously 
	 *  grabbed API data
	 *  @param apiFormat The time format the 3.0 Cyara API uses
	 *  @param comparedTimeframes All of the different date ranges 
	 *  we want our scatter graph to show
	 *  @param outputFile The HTML file metrics are outputted to
	 *  @see generateComparisonScatterHtml
	 *  @since 1.0
	 */
	public static void getComparisonStats(String jsonFile, SimpleDateFormat apiFormat, ArrayList<String> comparedTimeframes, String outputFile) {

		SimpleDateFormat javaFormat = new SimpleDateFormat("EEE MMM dd HH:mm:ss z yyyy");
		SimpleDateFormat chartTitleFormat = new SimpleDateFormat("yyyy MMM dd");

		StringBuilder scatterConnData = new StringBuilder();
		StringBuilder scatterInitData = new StringBuilder();

		//Build out the chart data for each time frame
		for (int timeframeIndex = 0; timeframeIndex < comparedTimeframes.size(); timeframeIndex++) {
			HashMap<Integer, Integer> connectionTimes = new HashMap<Integer, Integer>();
			HashMap<Integer, Integer> initiationTimes = new HashMap<Integer, Integer>();
			String startTitle = "";
			String endTitle = "";

			int[] totalsByDayArr = new int[7];
			int[] failsByDayArr = new int[7];
			int[] satsByDayArr = new int[7];
			int[] successesByDayArr = new int[7];

			String comparedTimeframe = comparedTimeframes.get(timeframeIndex);

			String fileLine = "";

			try (BufferedReader br = new BufferedReader(new FileReader(jsonFile))) {
				while ((fileLine = br.readLine()) != null) {
					JsonObject previousLine = convertAPIOutputToJson(fileLine);

					//Get array of all results for this file line
					JsonArray previousResultsArr = previousLine.getJsonArray("results");

					Date startDate = javaFormat.parse(comparedTimeframe.substring(0, comparedTimeframe.indexOf(",")));
					startTitle = chartTitleFormat.format(startDate);
					Date endDate = javaFormat.parse(comparedTimeframe.substring(comparedTimeframe.indexOf(",")+1, comparedTimeframe.length()));
					endTitle = chartTitleFormat.format(endDate);

					for (int resultIndex = 0; resultIndex < previousResultsArr.size(); resultIndex++) {
						//Get specific call
						JsonObject previousResult = previousResultsArr.getJsonObject(resultIndex);

						//Get the test result object of the call
						JsonObject callTestResult = previousResult.getJsonObject("testResult");

						//Get the result from the result object
						String callResult = callTestResult.getString("result");

						//Get the result detail from the result object
						String callResultDetail = callTestResult.getString("detail");

						String runDateString = previousResult.getString("runDate");

						if (!runDateString.contains(".")) {
							runDateString = runDateString.replace("Z", ".0000000Z");
						}

						Date runDate = apiFormat.parse(runDateString);

						long timeAfterTrackingStart = runDate.getTime() - startDate.getTime();
						long timeBeforeTrackingEnd = runDate.getTime() - endDate.getTime();

						if (timeAfterTrackingStart >= 0 && timeBeforeTrackingEnd <= 0) {
							int dayOfWeek = Instant.parse(runDateString).atZone(ZoneId.systemDefault()).getDayOfWeek().getValue();

							if (callResult.equals("Failed") && !callResultDetail.contains("Error(s) preventing start")) {
								failsByDayArr[dayOfWeek-1]++;
							}
							else if (callResult.equals("Satisfactory")) {
								satsByDayArr[dayOfWeek-1]++;
							}
							else if (callResult.equals("Success")) {
								successesByDayArr[dayOfWeek-1]++;
							}

							if (!callResult.equals("InternalError") && !callResultDetail.contains("Error(s) preventing start")) {
								totalsByDayArr[dayOfWeek-1]++;

								JsonObject testStepResultList = previousResult.getJsonObject("testStepResultList");

								String connectionTime = testStepResultList.getJsonObject("ringStepResult").getString("responseTime");
								connectionTimes.put(UtilsTime.getMillis(runDateString.substring(runDateString.indexOf("T")+1, runDateString.indexOf("Z"))), UtilsTime.getMillis(connectionTime));

								if (testStepResultList.getJsonArray("callStepResult").size() > 0) {
									String initiationTime = testStepResultList.getJsonArray("callStepResult").getJsonObject(0).getString("responseTime");

									int initTimeMillis = 0;
									if (UtilsTime.getMillis(initiationTime) > 0) {
										initTimeMillis = UtilsTime.getMillis(initiationTime);
									}
									else {
										if (testStepResultList.getJsonArray("callStepResult").size() > 1) {
											String backupInitiationTime = testStepResultList.getJsonArray("callStepResult").getJsonObject(1).getString("responseTime");
											initTimeMillis = UtilsTime.getMillis(backupInitiationTime);
										}
									}

									if (initTimeMillis > 0) {
										initiationTimes.put(UtilsTime.getMillis(runDateString.substring(runDateString.indexOf("T")+1, runDateString.indexOf("Z"))), initTimeMillis);
									}
								}
							}

						}
					}
				}
			}
			catch (IOException ex) {
				ex.printStackTrace();
			} catch (ParseException e) {
				e.printStackTrace();
			}

			//Build out data here, then append to dataset

			StringBuilder connectionTimePlots = new StringBuilder();
			StringBuilder initiationTimePlots = new StringBuilder();

			for (int i : connectionTimes.keySet()) {
				int connectionTimeOfDay = i;
				int connectionMillis = connectionTimes.get(i);
				connectionTimePlots.append("{\r\n" + 
						"                x: " + connectionTimeOfDay + ",\r\n" + 
						"                y: " + connectionMillis + "\r\n" + 
						"            }, ");
			}

			for (int i : initiationTimes.keySet()) {
				int initiationTimeOfDay = i;
				int initiationMillis = initiationTimes.get(i);

				initiationTimePlots.append("{\r\n" + 
						"                x: " + initiationTimeOfDay + ",\r\n" + 
						"                y: " + initiationMillis + "\r\n" + 
						"            }, ");
			}

			Random obj = new Random();
			int rand_num = obj.nextInt(0xffffff + 1);

			String connectionData = "{\r\n" + 
					"            label: 'Conn " + startTitle + " - " + endTitle + "',\r\n" + 
					"            data: [" + connectionTimePlots.toString() + "],\r\n" + 
					"			 borderColor: '" + String.format("#%06x", rand_num) + "',\r\n" + 
					"			 backgroundColor: '" + String.format("#%06x", rand_num) + "'\r\n" + 
					"        },\n";

			String initationData = "{\r\n" + 
					"            label: 'Init " + startTitle + " - " + endTitle + "',\r\n" + 
					"            data: [" + initiationTimePlots.toString() + "],\r\n" + 
					"			 borderColor: '" + String.format("#%06x", rand_num) + "',\r\n" + 
					"			 backgroundColor: '" + String.format("#%06x", rand_num) + "'\r\n" + 
					"        },\n";

			scatterConnData.append(connectionData);
			scatterInitData.append(initationData);
		}

		String connScatterHtml = generateComparisonScatterHtml(scatterConnData, "scatterConn", "Connection");
		String initScatterHtml = generateComparisonScatterHtml(scatterInitData, "scatterInit", "Initiation");

		Utils.logMessage("\n<br><br><br><br><br>\n" + connScatterHtml + "\n<br><br><br>\n" + initScatterHtml, outputFile, true);

	}


	/**
	 *  Create the HTML structure for a scatter graph. The scatter
	 *  graph generated is used for viewing results within one
	 *  or more date ranges.
	 *  
	 *  @param scatterData The JavaScript from data collection 
	 *  containing the graph data (time of day and response time)
	 *  @param canvasId The html id used to reference this chart
	 *  @param yLabelString The label on the chart to tell the 
	 *  user what this chart is showing
	 *  @return The JavaScript and HTML for the scatter chart
	 *  @see getComparisonStats
	 *  @since 1.0
	 */
	public static String generateComparisonScatterHtml(StringBuilder scatterData, String canvasId, String yLabelString) {
		return "<div style=\"width:75%;\">\r\n" + 
				"<canvas id=\"" + canvasId + "\" width=\"1400\" height=\"400\"></canvas>\r\n" + 
				"</div>\r\n" + 
				"<script>\r\n" + 
				"	var aftc = document.getElementById('" + canvasId + "');\r\n" + 
				"	var aftChart = new Chart(aftc, {\r\n" + 
				"	 type: 'scatter',\r\n" + 
				"    data: {\r\n" + 
				"        datasets: [" + scatterData.toString() + "]\r\n" + 
				"    },\r\n" + 
				"options: {\r\n" + 
				"        tooltips: {\r\n" + 
				"            callbacks: {\r\n" + 
				"                label: function(tooltipItem, data) {\r\n" + 
				"                    var xVal = tooltipItem.xLabel;\r\n" + 
				"\r\n" + 
				"                    var milliseconds = parseInt((xVal % 1000) / 100),\r\n" + 
				"                        seconds = Math.floor((xVal / 1000) % 60),\r\n" + 
				"                        minutes = Math.floor((xVal / (1000 * 60)) % 60),\r\n" + 
				"                        hours = Math.floor((xVal / (1000 * 60 * 60)) % 24);\r\n" + 
				"\r\n" + 
				"                    hours = (hours < 10) ? \"0\" + hours : hours;\r\n" + 
				"                    minutes = (minutes < 10) ? \"0\" + minutes : minutes;\r\n" + 
				"                    seconds = (seconds < 10) ? \"0\" + seconds : seconds;\r\n" + 
				"\r\n" + 
				"                    return \"(\" + hours + \":\" + minutes + \":\" + seconds + \", \" + tooltipItem.yLabel + \" ms)\";\r\n" + 
				"                },\r\n" + 
				"                title: function(tooltipItem, data) {\r\n" + 
				"                    return data.labels[tooltipItem[0].index];\r\n" + 
				"                }\r\n" + 
				"            }\r\n" + 
				"        },\r\n" + 
				"        scales: {\r\n" + 
				"            xAxes: [{\r\n" + 
				"                scaleLabel: {\r\n" + 
				"                    display: true,\r\n" + 
				"                    labelString: 'Time of Day'\r\n" + 
				"                },\r\n" + 
				"                ticks: {\r\n" + 
				"                    userCallback: function(label, index, labels) {\r\n" + 
				"                        var milliseconds = parseInt((label % 1000) / 100),\r\n" + 
				"                            seconds = Math.floor((label / 1000) % 60),\r\n" + 
				"                            minutes = Math.floor((label / (1000 * 60)) % 60),\r\n" + 
				"                            hours = Math.floor((label / (1000 * 60 * 60)) % 24);\r\n" + 
				"\r\n" + 
				"                        hours = (hours < 10) ? \"0\" + hours : hours;\r\n" + 
				"                        minutes = (minutes < 10) ? \"0\" + minutes : minutes;\r\n" + 
				"                        seconds = (seconds < 10) ? \"0\" + seconds : seconds;\r\n" + 
				"\r\n" + 
				"                        return hours + \":\" + minutes + \":\" + seconds;\r\n" + 
				"                    }\r\n" + 
				"                }\r\n" + 
				"            }],\r\n" + 
				"            yAxes: [{\r\n" + 
				"                scaleLabel: {\r\n" + 
				"                    display: true,\r\n" + 
				"                    labelString: '" + yLabelString + " Time - Millis'\r\n" + 
				"                }\r\n" + 
				"            }]\r\n" + 
				"        }\r\n" + 
				"    }" +
				"		});\r\n" + 
				"</script>\r\n";
	}


	/**
	 *  Prepares data and makes calls for all the charts
	 *  used to display data for the provided time frame.
	 *  <p>
	 *  Upon generation of the chart HTMLs, appends said HTML
	 *  to the output HTML file.
	 *  
	 *  @param jsonFile The source file of all the previously 
	 *  grabbed API data
	 *  @param apiFormat The time format the 3.0 Cyara API uses
	 *  @param startDate The first date in which data is retrieved, inclusive
	 *  @param endDate The last date in which data is retrieved, exclusive
	 *  @param timeframe The human friendly label for the chart telling the time period
	 *  @param baseUrl The core Cyara Portal URL
	 *  @param accountNum The account number in the portal referenced
	 *  @param outputFile The HTML file metrics are outputted to
	 *  @see generateDaysChartHtml
	 *  @see generatePieHtml
	 *  @see generateHoursChartHtml
	 *  @see generateTimesChartHtml
	 *  @since 1.0
	 */
	public static void getTimeframeStats(String jsonFile, SimpleDateFormat apiFormat, Date startDate, Date endDate, String timeframe, String baseUrl, int accountNum, String outputFile) {
		System.out.println("Getting timeframe stats for " + startDate + " to " + endDate);
		//Days will start on Monday, not Sunday, as per how the platform works

		//Arrays for calls by day chart
		int[] totalsByDayArr = new int[7];
		int[] failsByDayArr = new int[7];
		int[] satsByDayArr = new int[7];
		int[] successesByDayArr = new int[7];


		//Hashmaps for pie chart of failed test cases
		HashMap<String, Integer> totalTestCaseIds = new HashMap<String, Integer>();
		HashMap<String, Integer> failedTestCaseIds = new HashMap<String, Integer>();
		HashMap<String, Integer> satTestCaseIds = new HashMap<String, Integer>();
		HashMap<String, Integer> successTestCaseIds = new HashMap<String, Integer>();

		/*
		 * Hashmap for campaigns most associated with those failed test cases.
		 * Used for URL generation of linking the user to the worst campaign associated.
		 */
		HashMap<Integer, String> totalCampaignIds = new HashMap<Integer, String>();
		HashMap<Integer, String> failedCampaignIds = new HashMap<Integer, String>();
		HashMap<Integer, String> satCampaignIds = new HashMap<Integer, String>();
		HashMap<Integer, String> successCampaignIds = new HashMap<Integer, String>();

		//Arrays for calls per hour chart
		int[] totalsByHourArr = new int[24];
		int[] failsByHourArr = new int[24];
		int[] satsByHourArr = new int[24];
		int[] successesByHourArr = new int[24];

		//2D ArrayLists for connection and initiation calculations chart
		ArrayList<ArrayList<Integer>> connectionTimesList = new ArrayList<ArrayList<Integer>>();
		ArrayList<ArrayList<Integer>> initiationTimesList = new ArrayList<ArrayList<Integer>>();

		//Build the first dimension with 24 entries for each hour
		for (int lists = 0; lists < 24; lists++) {
			connectionTimesList.add(new ArrayList<Integer>());
			initiationTimesList.add(new ArrayList<Integer>());
		}

		String fileLine = "";

		//Loop through every grabbed row of pulse Json and parse datapoints
		try (BufferedReader br = new BufferedReader(new FileReader(jsonFile))) {
			while ((fileLine = br.readLine()) != null) {

				JsonObject previousLine = convertAPIOutputToJson(fileLine);

				//Get array of all results for this file line
				JsonArray previousResultsArr = previousLine.getJsonArray("results");

				for (int resultIndex = 0; resultIndex < previousResultsArr.size(); resultIndex++) {
					//Get specific call
					JsonObject previousResult = previousResultsArr.getJsonObject(resultIndex);

					JsonObject testCase = previousResult.getJsonObject("testCase");
					int testCaseId = testCase.getInt("testCaseId");
					String testCaseName = testCase.getString("name");

					JsonObject campaign = previousResult.getJsonObject("campaign");

					int campaignId = campaign.getInt("campaignId");

					//Get the test result object of the call
					JsonObject callTestResult = previousResult.getJsonObject("testResult");

					//Get the result from the result object
					String callResult = callTestResult.getString("result");

					//Get the result detail from the result object
					String callResultDetail = callTestResult.getString("detail");

					String runDateString = previousResult.getString("runDate");

					if (!runDateString.contains(".")) {
						runDateString = runDateString.replace("Z", ".0000000Z");
					}
					Date runDate = apiFormat.parse(runDateString);

					long timeAfterTrackingStart = runDate.getTime() - startDate.getTime();
					long timeBeforeTrackingEnd = runDate.getTime() - endDate.getTime();

					//Check that this call is within our date range
					if (timeAfterTrackingStart >= 0 && timeBeforeTrackingEnd <= 0) {
						int dayOfWeek = Instant.parse(runDateString).atZone(ZoneId.systemDefault()).getDayOfWeek().getValue();
						int hourOfDay = Instant.parse(runDateString).atZone(ZoneId.systemDefault()).getHour();

						if (callResult.equals("Failed") && !callResultDetail.contains("Error(s) preventing start")) {
							failsByDayArr[dayOfWeek-1]++;
							failedTestCaseIds.merge("(#" + testCaseId + ") " + testCaseName, 1, Integer::sum);
							failedCampaignIds.merge(campaignId, " " + testCaseId, String::concat);
							failsByHourArr[hourOfDay]++;
						}
						else if (callResult.equals("Satisfactory")) {
							satsByDayArr[dayOfWeek-1]++;
							satTestCaseIds.merge("(#" + testCaseId + ") " + testCaseName, 1, Integer::sum);
							satsByHourArr[hourOfDay]++;

							satCampaignIds.merge(campaignId, " " + testCaseId, String::concat);
						}
						else if (callResult.equals("Success")) {
							successesByDayArr[dayOfWeek-1]++;
							successTestCaseIds.merge("(#" + testCaseId + ") " + testCaseName, 1, Integer::sum);
							successesByHourArr[hourOfDay]++;

							successCampaignIds.merge(campaignId, " " + testCaseId, String::concat);
						}

						if (!callResult.equals("InternalError") && !callResultDetail.contains("Error(s) preventing start")) {
							totalsByDayArr[dayOfWeek-1]++;

							totalTestCaseIds.merge("(#" + testCaseId + ") " + testCaseName, 1, Integer::sum);
							totalsByHourArr[hourOfDay]++;

							totalCampaignIds.merge(campaignId, " " + testCaseId, String::concat);

							JsonObject testStepResultList = previousResult.getJsonObject("testStepResultList");

							String connectionTime = testStepResultList.getJsonObject("ringStepResult").getString("responseTime");
							connectionTimesList.get(hourOfDay).add(UtilsTime.getMillis(connectionTime));

							if (testStepResultList.getJsonArray("callStepResult").size() > 0) {
								String initiationTime = testStepResultList.getJsonArray("callStepResult").getJsonObject(0).getString("responseTime");

								int initTimeMillis = 0;
								if (UtilsTime.getMillis(initiationTime) > 0) {
									initTimeMillis = UtilsTime.getMillis(initiationTime);
								}
								else {
									if (testStepResultList.getJsonArray("callStepResult").size() > 1) {
										String backupInitiationTime = testStepResultList.getJsonArray("callStepResult").getJsonObject(1).getString("responseTime");
										initTimeMillis = UtilsTime.getMillis(backupInitiationTime);
									}
								}


								if (initTimeMillis > 0) {
									initiationTimesList.get(hourOfDay).add(initTimeMillis);
								}
							}
						}


					}
				}
			}
		}
		catch (IOException ex) {
			ex.printStackTrace();
			System.exit(1);
		} catch (ParseException e) {
			e.printStackTrace();
			System.exit(1);
		}


		String daysChartHtml = generateDaysChartHtml(totalsByDayArr, failsByDayArr, satsByDayArr, successesByDayArr, timeframe);

		String failedPieHtml = generatePieHtml("Failed", sortByValue(failedTestCaseIds), failedCampaignIds, UtilsTime.getUTCTicks(startDate), UtilsTime.getUTCTicks(endDate), baseUrl, accountNum, timeframe);

		String hoursChartHtml = generateHoursChartHtml(totalsByHourArr, failsByHourArr, satsByHourArr, successesByHourArr, timeframe);

		String responseTimesChartHtml = generateTimesChartHtml(connectionTimesList, initiationTimesList, timeframe);

		Utils.logMessage("<script src=\"Chart.bundle.js\"></script>\n" + daysChartHtml + "\n" + failedPieHtml + "\n" + hoursChartHtml + "\n<br><br><br><br><br>\n" + responseTimesChartHtml, outputFile, true);

	}


	/**
	 *  Prepares data and generates the HTML for the
	 *  hour by hour response time metrics chart. This 
	 *  data will be broken down into the min, max, mean, 
	 *  median, mode and 95 percentile.
	 *  
	 *  @param connectionTimesList The 2D ArrayList of all our 
	 *  connection times for every hour of the day
	 *  @param initiationTimesList The 2D ArrayList of all our 
	 *  initiation times for every hour of the day
	 *  @param timeframe The human friendly label for the chart telling the time period
	 *  @see getTimeframeStats
	 *  @see buildStatsDatasets
	 *  @return The HTML for the hourly response metrics chart
	 *  @since 1.0
	 */
	public static String generateTimesChartHtml(ArrayList<ArrayList<Integer>> connectionTimesList, ArrayList<ArrayList<Integer>> initiationTimesList, String timeframe) {
		int[][] connTimeStats = new int[24][6]; //min max mean median mode 95-percentile
		int[][] initTimeStats = new int[24][6];

		for (int x = 0; x < connectionTimesList.size(); x++) {
			ArrayList<Integer> thisHourConnections = connectionTimesList.get(x);
			Collections.sort(thisHourConnections);

			ArrayList<Integer> thisHourInitiations = initiationTimesList.get(x);
			Collections.sort(thisHourInitiations);

			if (thisHourConnections.size() > 0) {
				connTimeStats[x][0] = thisHourConnections.get(0); //Min
				initTimeStats[x][0] = thisHourInitiations.get(0); //Min

				connTimeStats[x][1] = thisHourConnections.get(thisHourConnections.size()-1); //Max
				initTimeStats[x][1] = thisHourInitiations.get(thisHourInitiations.size()-1); //Max

				connTimeStats[x][2] = UtilsMath.findMean(thisHourConnections); //Mean
				initTimeStats[x][2] = UtilsMath.findMean(thisHourInitiations); //Mean

				connTimeStats[x][3] = UtilsMath.findMedian(thisHourConnections); //Median
				initTimeStats[x][3] = UtilsMath.findMedian(thisHourInitiations); //Median

				connTimeStats[x][4] = UtilsMath.findMode(thisHourConnections); //Mode
				initTimeStats[x][4] = UtilsMath.findMode(thisHourInitiations); //Mode

				connTimeStats[x][5] = UtilsMath.findPercentile(thisHourConnections, 95.0); //Mode
				initTimeStats[x][5] = UtilsMath.findPercentile(thisHourInitiations, 95.0); //Mode
			}
			else {
				connTimeStats[x][0] = 0; //Min
				initTimeStats[x][0] = 0; //Min

				connTimeStats[x][1] = 0; //Max
				initTimeStats[x][1] = 0; //Max

				connTimeStats[x][2] = 0; //Mean
				initTimeStats[x][2] = 0; //Mean

				connTimeStats[x][3] = 0; //Median
				initTimeStats[x][3] = 0; //Median

				connTimeStats[x][4] = 0; //Mode
				initTimeStats[x][4] = 0; //Mode

				connTimeStats[x][5] = 0; //Mode
				initTimeStats[x][5] = 0; //Mode
			}
		}



		return "<div style=\"width:75%;\">\r\n" + 
		"<canvas id=\"responseChart\" width=\"1400\" height=\"400\"></canvas>\r\n" + 
		"</div>\r\n" + 
		"<script>\r\n" + 
		"	var aftc = document.getElementById('responseChart');\r\n" + 
		"	var aftChart = new Chart(aftc, {\r\n" + 
		"	type: 'line',\r\n" + 
		"			data: {\r\n" + 
		"	        labels: ['12am', '1am', '2am', '3am', '4am', '5am', '6am', '7am', '8am', '9am', '10am', '11am', '12pm', '1pm', '2pm', '3pm', '4pm', '5pm', '6pm', '7pm', '8pm', '9pm', '10pm', '11pm'],\r\n" + 
		"	        datasets: [\r\n" + 
		buildStatsDatasets(connTimeStats, "Conn") +
		buildStatsDatasets(initTimeStats, "Init") +
		"			]\r\n" + 
		"			},\r\n" + 
		"			options: {\r\n" + 
		"				title: {\r\n" + 
		"					text: 'Connection and Initiation Millis - " + timeframe + "'\r\n" + 
		"				},\r\n" + 
		"				scales: {\r\n" + 
		"					xAxes: [{\r\n" + 
		"						scaleLabel: {\r\n" + 
		"							display: true,\r\n" + 
		"							labelString: 'Time'\r\n" + 
		"						}\r\n" + 
		"					}],\r\n" + 
		"					yAxes: [{\r\n" + 
		"						scaleLabel: {\r\n" + 
		"							display: true,\r\n" + 
		"							labelString: 'Connection and Initiation Millis - " + timeframe + "'\r\n" + 
		"						}\r\n" + 
		"					}]\r\n" + 
		"				}\r\n" + 
		"			}\r\n" + 
		"		});\r\n" + 
		"</script>";

	}


	/**
	 *  Prepares data and generates the JavaScript for the
	 *  hour by hour response time metrics chart. This 
	 *  data will be broken down into the min, max, mean, 
	 *  median, mode and 95 percentile.
	 *  
	 *  @param timeStats The metrics of each hour of the day
	 *  @param dataType The human friendly label for data tracked
	 *  @see generateTimesChartHtml
	 *  @return The JavaScript for the hourly response metrics chart
	 *  @since 1.0
	 */
	public static String buildStatsDatasets(int[][] timeStats, String dataType) {
		String[] statMeasurements = {"Min", "Max", "Mean", "Median", "Mode", "95th Percentile"};
		StringBuilder dsBuilder = new StringBuilder();


		//Loop thru the various measurements
		for (int x = 0; x < timeStats[0].length; x++) {
			StringBuilder hourMillis = new StringBuilder();
			Random obj = new Random();
			int rand_num = obj.nextInt(0xffffff + 1);

			//Loop thru the hours for this measurement
			for (int y = 0; y < timeStats.length; y++) {
				hourMillis = hourMillis.append(timeStats[y][x] + ", ");
			}

			//We've now grabbed the hours for this measurement, let's build our dataset
			dsBuilder.append("				{\r\n" + 
					"				data: [" + hourMillis.substring(0, hourMillis.length()-2).toString() + "],\r\n" + 
					"				label: \"" + dataType + " " + statMeasurements[x] + "\",\r\n" + 
					"				borderColor: \"" + String.format("#%06x", rand_num) + "\",\r\n" + 
					"				fill: false\r\n" + 
					"				},\r\n");
		}

		return dsBuilder.toString();

	}

	/**
	 *  Prepares data and generates the HTML and 
	 *  JavaScript for the pie chart showing the 
	 *  top 10 most failed test cases in the 
	 *  given time frame.
	 *  
	 *  @param outcome The call result we're looking for
	 *  @param sortedTestCaseIds All the test cases found 
	 *  within the given time frame
	 *  @param campaignTCResultIds All the campaigns in the given 
	 *  time frame which had a failure
	 *  @param startNS The C# representation of the starting 
	 *  nanoseconds for our time frame
	 *  @param endNS The C# representation of the ending 
	 *  nanoseconds for our time frame
	 *  @param baseUrl The core Cyara Portal URL
	 *  @param accountNum The account number in the portal referenced
	 *  @param timeframe The human friendly label for the chart telling the time period
	 *  @see getTimeframeStats
	 *  @return The HTML and JavaScript for the pie chart
	 *  @since 1.0
	 */
	public static String generatePieHtml(String outcome, HashMap<String, Integer> sortedTestCaseIds, HashMap<Integer, String> campaignTCResultIds, long startNS, long endNS, String baseUrl, int accountNum, String timeframe) {
		ArrayList<PulseMetrics> mostFrequentTestCases = new ArrayList<PulseMetrics>();

		// Loop thru hashset to create PulseMetrics objects
		for (String i : sortedTestCaseIds.keySet()) {
			int id = Integer.parseInt(i.substring(i.indexOf("#")+1, i.indexOf(")")));
			String testCaseName = i.substring(i.indexOf(")")+1, i.length());

			//Create metric with test case ID, test case name, and failure count
			PulseMetrics myMetrics = new PulseMetrics(id, testCaseName, sortedTestCaseIds.get(i));
			mostFrequentTestCases.add(myMetrics);
		}


		StringBuilder labels = new StringBuilder();
		StringBuilder data = new StringBuilder();
		StringBuilder onClickCases = new StringBuilder();

		int failedTCsSize = mostFrequentTestCases.size() - 10;

		//Loop thru the top X most instances of the tracked result
		for (int index = mostFrequentTestCases.size()-1; index >= failedTCsSize; index--) {
			int tcId = mostFrequentTestCases.get(index).getTestCaseId();

			labels.append("\"(#" + tcId + ") " + mostFrequentTestCases.get(index).getTestCaseName() + "\", ");
			data.append(mostFrequentTestCases.get(index).getFailCount() + ", ");

			//Holders for which campaign had the most failures for the test case we're checking, and how many failures there were
			int mostCampaignId = 0;
			int mostCampaignCount = 0;

			//Loop thru the campaigns and their failures to find campaign with most failures for this TC
			for (Integer i : campaignTCResultIds.keySet()) {
				int campaignId = i;
				int campaignCount = 0;
				String[] campaignResults = campaignTCResultIds.get(i).trim().split(" ");
				for (int result = 0; result < campaignResults.length; result++) {
					if (Integer.parseInt(campaignResults[result]) == tcId) {
						campaignCount++;
					}
				}

				//This campaign had more failues for this TC, use it instead
				if (campaignCount > mostCampaignCount) {
					mostCampaignCount = campaignCount;
					mostCampaignId = campaignId;
				}

			}

			onClickCases.append("	      case '" + "(#" + tcId + ") " + mostFrequentTestCases.get(index).getTestCaseName() + "':\r\n" + 
					"	         window.open('" + baseUrl + "cyarawebportal/" + accountNum + "/report/pulsedetail?TestCaseId=" + tcId + "&CampaignId=" + mostCampaignId + "&DateRange=Custom&From=" + startNS + "&To=" + endNS + "');\r\n" + 
					"	         break;\r\n");
		}

		//Label which is on right hand of pie chart
		String labelsParsed = labels.toString();
		labelsParsed = labelsParsed.substring(0, labelsParsed.lastIndexOf(","));

		String dataParsed = data.toString();
		dataParsed = dataParsed.substring(0, dataParsed.lastIndexOf(","));

		return "<canvas id=\"failuresPie\" width=\"800\" height=\"800\"></canvas>\r\n" + 
		"<script>\r\n" + 
		"	var canvasP = document.getElementById(\"failuresPie\");\r\n" + 
		"	var ctxP = canvasP.getContext('2d');\r\n" + 
		"	var myPieChart = new Chart(ctxP, {\r\n" + 
		"	   type: 'pie',\r\n" + 
		"	   data: {\r\n" + 
		"	      labels: [" + labelsParsed + "],\r\n" + 
		"	      datasets: [{\r\n" + 
		"	         data: [" + dataParsed + "],\r\n" + 
		"	            backgroundColor: [\r\n" + 
		"	                'rgba(255, 51, 0, 0.5)',\r\n" + 
		"	                'rgba(255, 102, 0, 0.5)',\r\n" + 
		"	                'rgba(255, 153, 102, 0.5)',\r\n" + 
		"	                'rgba(255, 255, 0, 0.5)',\r\n" + 
		"	                'rgba(171, 219, 119, 0.5)',\r\n" + 
		"	                'rgba(0, 176, 80, 0.5)',\r\n" + 
		"					'rgba(183, 222, 232, 0.5)',\r\n" + 
		"					'rgba(0, 102, 255, 0.5)',\r\n" + 
		"					'rgba(204, 0, 204, 0.5)',\r\n" + 
		"					'rgba(153, 0, 204, 0.5)'\r\n" + 
		"	            ],\r\n" + 
		"	            borderColor: [\r\n" + 
		"	                'rgba(255, 51, 0, 1)',\r\n" + 
		"	                'rgba(255, 102, 0, 1)',\r\n" + 
		"	                'rgba(255, 153, 102, 1)',\r\n" + 
		"	                'rgba(255, 255, 0, 1)',\r\n" + 
		"	                'rgba(171, 219, 119, 1)',\r\n" + 
		"	                'rgba(0, 176, 80, 1)',\r\n" + 
		"					'rgba(183, 222, 232, 1)',\r\n" + 
		"					'rgba(0, 102, 255, 1)',\r\n" + 
		"					'rgba(204, 0, 204, 1)',\r\n" + 
		"					'rgba(153, 0, 204, 1)'\r\n" + 
		"	            ],\r\n" + 
		"	         hoverBackgroundColor: ['rgba(255, 51, 0, 1)',\r\n" + 
		"	                'rgba(255, 102, 0, 1)',\r\n" + 
		"	                'rgba(255, 153, 102, 1)',\r\n" + 
		"	                'rgba(255, 255, 0, 1)',\r\n" + 
		"	                'rgba(171, 219, 119, 1)',\r\n" + 
		"	                'rgba(0, 176, 80, 1)',\r\n" + 
		"					'rgba(183, 222, 232, 1)',\r\n" + 
		"					'rgba(0, 102, 255, 1)',\r\n" + 
		"					'rgba(204, 0, 204, 1)',\r\n" + 
		"					'rgba(153, 0, 204, 1)']\r\n" + 
		"	      }]\r\n" + 
		"	   },\r\n" + 
		"	   options: {\r\n" + 
		"			responsive: false,\r\n" + 
		"			legend: {\r\n" + 
		"				display: true,\r\n" + 
		"				position: \"right\"\r\n" + 
		"			},\r\n" + 
		"			title: {\r\n" + 
		"				display: true,\r\n" + 
		"				position: \"left\",\r\n" + 
		"				text: 'Top 10 " + outcome + " Test Cases - " + timeframe + "'\r\n" + 
		"			},\r\n" + 
		"	 tooltips: {\r\n" + 
		"	    callbacks: {\r\n" + 
		"	      label: function(tooltipItem, data) {\r\n" + 
		"	        var dataset = data.datasets[tooltipItem.datasetIndex];\r\n" + 
		"	        var meta = dataset._meta[Object.keys(dataset._meta)[0]];\r\n" + 
		"	        var total = meta.total;\r\n" + 
		"	        var currentValue = dataset.data[tooltipItem.index];\r\n" + 
		"	        var percentage = parseFloat((currentValue/total*100).toFixed(1));\r\n" + 
		"	        return currentValue + ' (' + percentage + '%)';\r\n" + 
		"	      },\r\n" + 
		"	      title: function(tooltipItem, data) {\r\n" + 
		"	        return data.labels[tooltipItem[0].index];\r\n" + 
		"	      }\r\n" + 
		"	    }\r\n" + 
		"	  }\r\n" + 
		"	   }\r\n" + 
		"	});\r\n" + 
		"	\r\n" + 
		"	canvasP.onclick = function(e) {\r\n" + 
		"	   var slice = myPieChart.getElementAtEvent(e);\r\n" + 
		"	   var label = slice[0]._model.label;\r\n" + 
		"	   switch (label) {\r\n" + 
		onClickCases.toString() +
		"	   }\r\n" + 
		"	}\r\n" + 
		"</script>";
	}


	/**
	 *  Prepares data and generates the HTML and 
	 *  JavaScript for the hour by hour calls
	 *  made number categorized by overall result.
	 *  
	 *  @param totalsByHourArr The amount of total calls that 
	 *  happened for each hour of the time frame
	 *  @param failsByHourArr The amount of failed calls that 
	 *  happened for each hour of the time frame
	 *  @param satsByHourArr The amount of satisfactory calls that 
	 *  happened for each hour of the time frame
	 *  @param successesByHourArr The amount of successful calls that 
	 *  happened for each hour of the time frame
	 *  @param timeframe The human friendly label for the chart telling the time period
	 *  @see getTimeframeStats
	 *  @return The HTML and JavaScript for the calls per hour chart
	 *  @since 1.0
	 */
	public static String generateHoursChartHtml(int[] totalsByHourArr, int[] failsByHourArr, int[] satsByHourArr,
			int[] successesByHourArr, String timeframe) {
		String totalsData = String.valueOf(totalsByHourArr[0]);
		for (int index = 1; index < totalsByHourArr.length; index++) {
			totalsData = totalsData + ", " + totalsByHourArr[index];
		}

		String failsData = String.valueOf(failsByHourArr[0]);
		for (int index = 1; index < failsByHourArr.length; index++) {
			failsData = failsData + ", " + failsByHourArr[index];
		}

		String satsData = String.valueOf(satsByHourArr[0]);
		for (int index = 1; index < satsByHourArr.length; index++) {
			satsData = satsData + ", " + satsByHourArr[index];
		}

		String successesData = String.valueOf(successesByHourArr[0]);
		for (int index = 1; index < successesByHourArr.length; index++) {
			successesData = successesData + ", " + successesByHourArr[index];
		}


		return "<div style=\"width:75%;\">\r\n" + 
		"<canvas id=\"timesChart\" width=\"1400\" height=\"400\"></canvas>\r\n" + 
		"</div>\r\n" + 
		"<script>\r\n" + 
		"	var aftc = document.getElementById('timesChart');\r\n" + 
		"	var aftChart = new Chart(aftc, {\r\n" + 
		"	type: 'line',\r\n" + 
		"			data: {\r\n" + 
		"	        labels: ['12am', '1am', '2am', '3am', '4am', '5am', '6am', '7am', '8am', '9am', '10am', '11am', '12pm', '1pm', '2pm', '3pm', '4pm', '5pm', '6pm', '7pm', '8pm', '9pm', '10pm', '11pm'],\r\n" + 
		"	        datasets: [\r\n" + 
		/*"				{\r\n" + 
		"				data: [" + totalsData + "],\r\n" + 
		"				label: \"Total\",\r\n" + 
		"				borderColor: \"#289fc9\",\r\n" + 
		"				fill: false\r\n" + 
		"				},\r\n" + */
		"				{\r\n" + 
		"				data: [" + failsData + "],\r\n" + 
		"				label: \"Failed\",\r\n" + 
		"				borderColor: \"#ee1236\",\r\n" + 
		"				fill: false\r\n" + 
		"				},\r\n" + 
		"				{\r\n" + 
		"				data: [" + satsData + "],\r\n" + 
		"				label: \"Satisfactory\",\r\n" + 
		"				borderColor: \"#ff9e3b\",\r\n" + 
		"				fill: false\r\n" + 
		"				},\r\n" + 
		"				{\r\n" + 
		"				data: [" + successesData + "],\r\n" + 
		"				label: \"Success\",\r\n" + 
		"				borderColor: \"#5cc242\",\r\n" + 
		"				fill: false\r\n" + 
		"				}\r\n" + 
		"			]\r\n" + 
		"			},\r\n" + 
		"			options: {\r\n" + 
		"				title: {\r\n" + 
		"					text: 'Calls by Hour - " + timeframe + "'\r\n" + 
		"				},\r\n" + 
		"				scales: {\r\n" + 
		"					xAxes: [{\r\n" + 
		"						scaleLabel: {\r\n" + 
		"							display: true,\r\n" + 
		"							labelString: 'Time'\r\n" + 
		"						}\r\n" + 
		"					}],\r\n" + 
		"					yAxes: [{\r\n" + 
		"						scaleLabel: {\r\n" + 
		"							display: true,\r\n" + 
		"							labelString: 'Calls by Hour - " + timeframe + "'\r\n" + 
		"						}\r\n" + 
		"					}]\r\n" + 
		"				}\r\n" + 
		"			}\r\n" + 
		"		});\r\n" + 
		"</script>";
	}

	/**
	 *  Generates the HTML and 
	 *  JavaScript for the day by day calls
	 *  made number categorized by overall result.
	 *  <p>
	 *  The days are shown in a column chart and start
	 *  on Monday.
	 *  
	 *  @param totalsByDayArr The amount of total calls that 
	 *  happened for each hour of the time frame
	 *  @param failsByDayArr The amount of failed calls that 
	 *  happened for each hour of the time frame
	 *  @param satsByDayArr The amount of satisfactory calls that 
	 *  happened for each hour of the time frame
	 *  @param successesByDayArr The amount of successful calls that 
	 *  happened for each hour of the time frame
	 *  @param timeframe The human friendly label for the chart telling the time period
	 *  @see getTimeframeStats
	 *  @return The HTML and JavaScript for the calls per day chart
	 *  @since 1.0
	 */
	public static String generateDaysChartHtml(int[] totalsByDayArr, int[] failsByDayArr, int[] satsByDayArr,
			int[] successesByDayArr, String timeframe) {
		return "<canvas id=\"callsChart\" width=\"1400\" height=\"400\"></canvas>\r\n" + 
				"<script>\r\n" + 
				"	var ctx = document.getElementById('callsChart');\r\n" + 
				"	var callsChart = new Chart(ctx, {\r\n" + 
				"	    type: 'bar',\r\n" + 
				"	    data: {\r\n" + 
				"	        labels: ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'],\r\n" + 
				"	        datasets: [\r\n" + /*
								"			{\r\n" + 
				"	            label: 'All Calls by Day',\r\n" + 
				"	            data: [" + totalsByDayArr[0] + ", " + totalsByDayArr[1] + ", " + totalsByDayArr[2] + ", " + totalsByDayArr[3] +
				", " + totalsByDayArr[4] + ", " + totalsByDayArr[5] + ", " + totalsByDayArr[6] + "],\r\n" + 
				"	            backgroundColor: [\r\n" + 
				"	                'rgba(40, 159, 201, 0.2)',\r\n" + 
				"	                'rgba(40, 159, 201, 0.2)',\r\n" + 
				"	                'rgba(40, 159, 201, 0.2)',\r\n" + 
				"	                'rgba(40, 159, 201, 0.2)',\r\n" + 
				"	                'rgba(40, 159, 201, 0.2)',\r\n" + 
				"	                'rgba(40, 159, 201, 0.2)',\r\n" + 
				"					'rgba(40, 159, 201, 0.2)'\r\n" + 
				"	            ],\r\n" + 
				"	            borderColor: [\r\n" + 
				"	                'rgba(40, 159, 201, 1)',\r\n" + 
				"	                'rgba(40, 159, 201, 1)',\r\n" + 
				"	                'rgba(40, 159, 201, 1)',\r\n" + 
				"	                'rgba(40, 159, 201, 1)',\r\n" + 
				"	                'rgba(40, 159, 201, 1)',\r\n" + 
				"	                'rgba(40, 159, 201, 1)',\r\n" + 
				"					'rgba(40, 159, 201, 1)'\r\n" + 
				"	            ],\r\n" + 
				"	            borderWidth: 1\r\n" + 
				"	        },\r\n" + */
				"			{\r\n" + 
				"	            label: 'All Failures by Day',\r\n" + 
				"	            data: [" + failsByDayArr[0] + ", " + failsByDayArr[1] + ", " + failsByDayArr[2] + ", " + failsByDayArr[3] +
				", " + failsByDayArr[4] + ", " + failsByDayArr[5] + ", " + failsByDayArr[6] + "],\r\n" + 
				"	            backgroundColor: [\r\n" + 
				"	                'rgba(238, 18, 54, 0.2)',\r\n" + 
				"	                'rgba(238, 18, 54, 0.2)',\r\n" + 
				"	                'rgba(238, 18, 54, 0.2)',\r\n" + 
				"	                'rgba(238, 18, 54, 0.2)',\r\n" + 
				"	                'rgba(238, 18, 54, 0.2)',\r\n" + 
				"	                'rgba(238, 18, 54, 0.2)',\r\n" + 
				"					'rgba(238, 18, 54, 0.2)'\r\n" + 
				"	            ],\r\n" + 
				"	            borderColor: [\r\n" + 
				"	                'rgba(238, 18, 54, 1)',\r\n" + 
				"	                'rgba(238, 18, 54, 1)',\r\n" + 
				"	                'rgba(238, 18, 54, 1)',\r\n" + 
				"	                'rgba(238, 18, 54, 1)',\r\n" + 
				"	                'rgba(238, 18, 54, 1)',\r\n" + 
				"	                'rgba(238, 18, 54, 1)',\r\n" + 
				"					'rgba(238, 18, 54, 1)'\r\n" + 
				"	            ],\r\n" + 
				"	            borderWidth: 1\r\n" + 
				"	        },\r\n" + 
				"			{\r\n" + 
				"	            label: 'All Satisfactories by Day',\r\n" + 
				"	            data: [" + satsByDayArr[0] + ", " + satsByDayArr[1] + ", " + satsByDayArr[2] + ", " + satsByDayArr[3] +
				", " + satsByDayArr[4] + ", " + satsByDayArr[5] + ", " + satsByDayArr[6] + "],\r\n" + 
				"	            backgroundColor: [\r\n" + 
				"	                'rgba(255, 158, 59, 0.2)',\r\n" + 
				"	                'rgba(255, 158, 59, 0.2)',\r\n" + 
				"	                'rgba(255, 158, 59, 0.2)',\r\n" + 
				"	                'rgba(255, 158, 59, 0.2)',\r\n" + 
				"	                'rgba(255, 158, 59, 0.2)',\r\n" + 
				"	                'rgba(255, 158, 59, 0.2)',\r\n" + 
				"					'rgba(255, 158, 59, 0.2)'\r\n" + 
				"	            ],\r\n" + 
				"	            borderColor: [\r\n" + 
				"	                'rgba(255, 158, 59, 1)',\r\n" + 
				"	                'rgba(255, 158, 59, 1)',\r\n" + 
				"	                'rgba(255, 158, 59, 1)',\r\n" + 
				"	                'rgba(255, 158, 59, 1)',\r\n" + 
				"	                'rgba(255, 158, 59, 1)',\r\n" + 
				"	                'rgba(255, 158, 59, 1)',\r\n" + 
				"					'rgba(255, 158, 59, 1)'\r\n" + 
				"	            ],\r\n" + 
				"	            borderWidth: 1\r\n" + 
				"	        },\r\n" + 
				"			{\r\n" + 
				"	            label: 'All Successes by Day',\r\n" + 
				"	            data: [" + successesByDayArr[0] + ", " + successesByDayArr[1] + ", " + successesByDayArr[2] + ", " + successesByDayArr[3] +
				", " + successesByDayArr[4] + ", " + successesByDayArr[5] + ", " + successesByDayArr[6] + "],\r\n" + 
				"	            backgroundColor: [\r\n" + 
				"	                'rgba(92, 194, 66, 0.2)',\r\n" + 
				"	                'rgba(92, 194, 66, 0.2)',\r\n" + 
				"	                'rgba(92, 194, 66, 0.2)',\r\n" + 
				"	                'rgba(92, 194, 66, 0.2)',\r\n" + 
				"	                'rgba(92, 194, 66, 0.2)',\r\n" + 
				"	                'rgba(92, 194, 66, 0.2)',\r\n" + 
				"					'rgba(92, 194, 66, 0.2)'\r\n" + 
				"	            ],\r\n" + 
				"	            borderColor: [\r\n" + 
				"	                'rgba(92, 194, 66, 1)',\r\n" + 
				"	                'rgba(92, 194, 66, 1)',\r\n" + 
				"	                'rgba(92, 194, 66, 1)',\r\n" + 
				"	                'rgba(92, 194, 66, 1)',\r\n" + 
				"	                'rgba(92, 194, 66, 1)',\r\n" + 
				"	                'rgba(92, 194, 66, 1)',\r\n" + 
				"					'rgba(92, 194, 66, 1)'\r\n" + 
				"	            ],\r\n" + 
				"	            borderWidth: 1\r\n" + 
				"	        },\r\n" + 
				"			\r\n" + 
				"			]\r\n" + 
				"	    },\r\n" + 
				"	    options: {\r\n" + 
				"			responsive: false,\r\n" + 
				"	        scales: {\r\n" + 
				"	            yAxes: [{\r\n" + 
				"	                ticks: {\r\n" + 
				"	                    beginAtZero: true\r\n" + 
				"	                }\r\n" + 
				"	            }]\r\n" + 
				"	        },\r\n" + 
				"			title: {\r\n" + 
				"				display: true,\r\n" + 
				"				text: 'Call Results By Day - " + timeframe + "'\r\n" + 
				"			}\r\n" + 
				"	    }\r\n" + 
				"	});\r\n" + 
				"</script>";
	}


	/**
	 *  Parse the source file with all our
	 *  previously grabbed results, and find the
	 *  newest, most recent call grabbed.
	 *  
	 *  @param jsonFile The source file of all the previously 
	 *  grabbed API data
	 *  @return The most recent Test Result Id
	 *  @since 1.0
	 */
	public static int getLastTestResultId(String jsonFile) {
		String lastLine = "";
		int lastTestResultId = -1;
		String fileLine = "";

		//Read in the Json file of previously grabbed Pulse calls
		try (BufferedReader br = new BufferedReader(new FileReader(jsonFile))) {
			while ((fileLine = br.readLine()) != null) {
				lastLine = fileLine;
			}
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}

		try {
			//Grab very last row
			JsonObject lastResults = convertAPIOutputToJson(lastLine);

			JsonArray lastResultsArr = lastResults.getJsonArray("results");

			//Grabs the very last call of the very last row
			JsonObject lastTestResult = lastResultsArr.getJsonObject(lastResultsArr.size()-1);

			lastTestResultId = lastTestResult.getInt("testResultId");

			return lastTestResultId;
		}
		catch (JsonParsingException | IndexOutOfBoundsException ex) {
			return 1;
		}
	}


	/**
	 *  Convert a String object with Json format into
	 *  a JsonObject
	 *  
	 *  @param apiString The Json string to convert into Json
	 *  @return The JsonObject representation of the input String
	 *  @since 1.0
	 */
	public static JsonObject convertAPIOutputToJson(String apiString) {

		JsonReader apiReader = Json.createReader(new StringReader(apiString));
		JsonObject apiObject = apiReader.readObject();
		apiReader.close();

		return apiObject;
	}


	/**
	 *  Use the 2.5 API to dynamically generate a fresh 
	 *  3.0 API key:token pair.
	 *  
	 *  @param baseUrl The core Cyara Portal URL
	 *  @param authStringEnc The base64 encoded username and password
	 *  @return The raw API output
	 *  @since 1.0
	 */
	public static String generateKeyToken(String baseUrl, String authStringEnc) {
		String apiCall = baseUrl + "CyaraWebPortal/API/2.5/" + "user/apitoken";
		String resultLine = "";

		String tokenOutput = "";

		try {

			URL apiURL = new URL(apiCall);
			HttpsURLConnection importCon = (HttpsURLConnection) apiURL.openConnection();
			importCon.setRequestMethod("POST");
			importCon.setRequestProperty("Authorization", "Basic " + authStringEnc);
			importCon.setRequestProperty("Content-Type", "application/json");
			importCon.setRequestProperty("Accept", "application/json");
			importCon.setDoOutput(true);
			OutputStreamWriter writer = new OutputStreamWriter(importCon.getOutputStream(), "UTF-8");
			writer.write("");
			writer.close();
			BufferedReader importBR = new BufferedReader(new InputStreamReader(importCon.getInputStream()));

			while ((resultLine = importBR.readLine()) != null) {
				tokenOutput = tokenOutput + resultLine;
			}

			//status = importCon.getResponseCode();
			importBR.close();
		}
		catch (IOException ex) {
			StringWriter sw = new StringWriter();
			ex.printStackTrace(new PrintWriter(sw));
			String exceptionAsString = sw.toString();
			if (exceptionAsString.contains("401")) {
				tokenOutput = "401";
			}
			else if (exceptionAsString.contains("Connection refused")) {
				tokenOutput = "Connection refused";
			}
			else if (exceptionAsString.contains("UnknownHostException")) {
				tokenOutput = "Please check your internet connection and your url";
			}
			ex.printStackTrace();
		}

		return tokenOutput;
	}


	/**
	 *  Sort hashmap by values from lowest to highest.
	 *  
	 *  @param failedTestCaseIds The unsorted hashmap containing 
	 *  the names of each failed test case and their failure count
	 *  @return The sorted hashmap
	 *  @since 1.0
	 */
	public static HashMap<String, Integer> sortByValue(HashMap<String, Integer> failedTestCaseIds) 
	{ 
		// Create a list from elements of HashMap 
		List<Map.Entry<String, Integer> > list = 
				new LinkedList<Map.Entry<String, Integer> >(failedTestCaseIds.entrySet()); 

		// Sort the list 
		Collections.sort(list, new Comparator<Map.Entry<String, Integer> >() { 
			public int compare(Map.Entry<String, Integer> o1,  
					Map.Entry<String, Integer> o2) 
			{ 
				return (o1.getValue()).compareTo(o2.getValue()); 
			} 
		}); 

		// put data from sorted list to hashmap  
		HashMap<String, Integer> temp = new LinkedHashMap<String, Integer>(); 
		for (Map.Entry<String, Integer> aa : list) { 
			temp.put(aa.getKey(), aa.getValue()); 
		} 
		return temp;
	}


	/**
	 *  Compares two fail counts and checks equality.
	 *  
	 *  @param o The provided PuseMetric failure object
	 *  @return The equality check
	 *  @since 1.0
	 */
	@Override
	public int compareTo(PulseMetrics o) {
		return Integer.compare(this.problemTestCaseFailCount, o.problemTestCaseFailCount);
	}

	public PulseMetrics(int failTCId, String failTCName, int failCount) {
		problemTestCaseId  = failTCId;
		problemTestCaseName = failTCName;
		problemTestCaseFailCount = failCount;
	}


	/**
	 *  Get the problematic test case ID
	 *  
	 *  @return The test case ID
	 *  @since 1.0
	 */
	private int getTestCaseId() {
		return this.problemTestCaseId;
	}


	/**
	 *  Get the problematic test case Name
	 *  
	 *  @return The test case name
	 *  @since 1.0
	 */
	private String getTestCaseName() {
		return this.problemTestCaseName;
	}


	/**
	 *  Get the problematic test case failure count
	 *  
	 *  @return The failure count
	 *  @since 1.0
	 */
	private int getFailCount() {
		return this.problemTestCaseFailCount;
	}

	/*
	private int getProblemCampaign() {
		return this.problemCampaign;
	}

	private void setTestCaseId(int failTCId) {
		this.problemTestCaseId = failTCId;
	}

	private void setTestCaseName(String failTCName) {
		this.problemTestCaseName = failTCName;
	}

	private void setFailCount(int failCount) {
		this.problemTestCaseFailCount = failCount;
	}

	private void setProblemCampaign(int campaignId) {
		this.problemCampaign = campaignId;
	}
	 */

}
