package com.cyara.apisamples;

import java.util.ArrayList;

import javax.json.JsonObject;

import com.cyara.utilities.Utils;

/**
 * This program allows you to fire off consecutive Cyara test
 * cases. After all your specified test cases have run, the
 * program will print out the results, highlighting any runs
 * that failed.
 * <p>
 * This is not an official Cyara product and is not officially supported.
 * It is used for demonstration and training purposes. For assistance,
 * contact chris.ryan@cyara.com
 * 
 * @author Chris Ryan
 * @version 1.1
 */
public class TestCaseValidator {
	public static void main(String[] argv) {
		ApiFunctionalities connInfo = new ApiFunctionalities();

		int[] testCaseIDs = {17822304,19623948}; //Your test case IDs. Change as needed. For more than one, comma separate them with no spaces.

		ArrayList<String> ticketList = new ArrayList<>();

		//Loop thru and run all the entered test cases
		for (int x = 0; x < testCaseIDs.length; x++) {
			int testCaseId = testCaseIDs[x];

			JsonObject testCaseExecAck = connInfo.executeTestCase(testCaseId);
			ticketList.add(testCaseExecAck.getString("testRunTicketId"));

			Utils.sleepMode(180000); //How long to sleep between runs in milliseconds, change as needed.
		}

		//Print out all the tickets for debugging purposes
		for (int x = 0; x < ticketList.size(); x++) {
			System.out.println("Ticket: " + ticketList.get(x));
		}

		//All test cases have been run, start grabbing results
		for (int x = 0; x < ticketList.size(); x++) {
			JsonObject testCaseResult = connInfo.getTestCaseResultByTicketId(ticketList.get(x));
			System.out.println(testCaseResult.toString());

			JsonObject testCaseInfo = testCaseResult.getJsonObject("testCase");

			int testCaseId = testCaseInfo.getInt("testCaseId");

			String testCaseName = testCaseInfo.getString("name");

			String folderPath = "\\";
			if (!testCaseInfo.isNull("folderPath")) {
				folderPath = testCaseInfo.getString("folderPath");
			}

			int testResultId = testCaseResult.getInt("testResultId");

			JsonObject testResult = testCaseResult.getJsonObject("testResult");
			String result = testResult.getString("result");

			//If the overall test case result was not Success, print all the info
			if (!result.equals("Success")) {
				String detail = testResult.getString("detail").trim();

				System.out.print("This test case was " + result + " because of " + detail + ": " + folderPath + " | " + testCaseName + " | " + testCaseId + " | " + testResultId);
			}

		}
	}
}
