package com.cyara.utilities;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

/**
 * Various methods used throughout the samples for generic operations.
 * <p>
 * This is not an official Cyara product and is not officially supported.
 * It is used for demonstration and training purposes. For assistance,
 * contact chris.ryan@cyara.com
 * 
 * @author Chris Ryan
 * @version 1.0
 */
public class Utils {


	/**
	 * Instantiates a new Utils. Private to prevent instantiation.
	 * @version 1.0
	 */
	private Utils() {
		throw new IllegalStateException("Utility class");
	}


	/**
	 *  Appends a String to a destination file.
	 *  <p>
	 *  Adds a newline automatically to the end of the file.
	 *  
	 *  @param message The string we created that 
	 *  will be appended to the file
	 *  @param file The destination file where 
	 *  our string will be appended
	 *  @param removeNewLines Flag whether or not to strip newlines from message
	 *  @return The boolean if the output was successful
	 *  @since 1.0
	 */
	public static boolean logMessage(String message, String file, boolean removeNewLines) {
		File resultFile = new File(file);

		try {
			if (removeNewLines) {
				while (message.contains("\r") || message.contains("\n") || message.contains("\r\n") || message.contains("  ")) {
					message = message.replace("\r\n", " ").replace("\r", " ").replace("\n", " ").replace("  ", " ");
				}
			}
			FileWriter resultWriter = new FileWriter(resultFile, true);
			resultWriter.append(message + "\n");
			resultWriter.flush();
			resultWriter.close();

			return true;
		}
		catch (IOException ex) {
			ex.printStackTrace();
			System.exit(1);
			return false;
		}

	}


	/**
	 * Gets the index of an element in a String array.
	 * <p>
	 * Used throughout the samples when processing CSVs
	 * @param headerLineArr The String array of headers
	 * @param desiredHeader The specific header whose index we look for
	 * @return The index our header is located at in the header row
	 * @since 1.0
	 */
	public static int getHeaderIndex(String[] headerLineArr, String desiredHeader) {
		for (int x = 0; x < headerLineArr.length; x++) {
			String header = headerLineArr[x];
			if (header.equals(desiredHeader)) {
				return x;
			}
		}

		return -1;
	}


	/**
	 * Pauses the current thread for the specified number of milliseconds.
	 * 
	 * @param pauseLen The length of time to pause in milliseconds
	 * @since 1.0
	 */
	public static void sleepMode(int pauseLen) {
		try {
			Thread.sleep(pauseLen); }
		catch(InterruptedException ex) {
			Thread.currentThread().interrupt(); }
	}

}
